@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: AddMedia.py 6977 2006-07-01 04:55:36Z dallingham $
+# $Id: AddMedia.py 7980 2007-01-25 05:37:24Z dallingham $
 
 """
 Provides the interface to allow a person to add a media object to the database.
@@ -30,6 +30,7 @@ Provides the interface to allow a person to add a media object to the database.
 #
 #-------------------------------------------------------------------------
 import os
+import sys
 
 #-------------------------------------------------------------------------
 #
@@ -130,7 +131,8 @@ class AddMediaObject(ManagedWindow.ManagedWindow):
             mobj.set_handle(Utils.create_id())
             mobj.set_mime_type(None)
         else:
-            filename = self.file_text.get_filename()
+            filename = unicode(self.file_text.get_filename(), 
+                               sys.getfilesystemencoding())
             full_file = filename
 
             if self.relpath.get_active():
@@ -139,8 +141,6 @@ class AddMediaObject(ManagedWindow.ManagedWindow):
                     p = os.path.dirname(p)
                 filename = Utils.relative_path(filename,p)
 
-            print os.getcwd(), filename, full_file
-
             if os.path.exists(filename) == 0:
                 msgstr = _("Cannot import %s")
                 msgstr2 = _("The filename supplied could not be found.")
@@ -173,7 +173,10 @@ class AddMediaObject(ManagedWindow.ManagedWindow):
         see if the file exists. If it does, the imgae is loaded into
         the preview window.
         """
-        filename = unicode(self.file_text.get_filename())
+        fn = self.file_text.get_filename()
+        if not fn:
+            return
+        filename = unicode(fn, sys.getfilesystemencoding())
         basename = os.path.basename(filename)
         (root,ext) = os.path.splitext(basename)
         old_title  = unicode(self.description.get_text())
