@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ArgHandler.py 7325 2006-09-24 04:37:59Z dallingham $
+# $Id: ArgHandler.py 7980 2007-01-25 05:37:24Z dallingham $
 
 # Written by Alex Roitman
 
@@ -39,6 +39,7 @@ import gtk
 #
 #-------------------------------------------------------------------------
 import os
+import sys
 import getopt
 from gettext import gettext as _
 
@@ -152,6 +153,11 @@ class ArgHandler:
                     format = 'gramps-xml'
                 elif ftype == const.app_gramps:
                     format = 'grdb'
+                elif ftype in [const.app_gramps_package,const.app_geneweb]:
+                    print 'Unsupported type: "%s" for input file: %s' \
+                          % (ftype,fname)
+                    print 'Please create a new GRAMPS database and import the file.'
+                    continue
                 else:
                     print 'Unrecognized type: "%s" for input file: %s' \
                           % (ftype,fname)
@@ -320,7 +326,7 @@ class ArgHandler:
                     os._exit(1)
                 elif filetype == const.app_gramps_package:
                     print "Type: GRAMPS package"
-                    self.read_pkg(filename)
+                    self.vm.import_pkg(filename)
                 success = True
             else:
                 print "Unknown file type: %s" % filetype
@@ -429,8 +435,9 @@ class ArgHandler:
 
         elif Config.get(Config.RECENT_FILE) and Config.get(Config.AUTOLOAD):
             rf = Config.get(Config.RECENT_FILE)
-            filetype = Mime.get_type(rf)
-            self.vm.read_recent_file(rf,filetype)
+            if os.path.isfile(rf):
+                filetype = Mime.get_type(rf)
+                self.vm.read_recent_file(rf,filetype)
 
     #-------------------------------------------------------------------------
     #
@@ -625,7 +632,6 @@ class ArgHandler:
         elif action == 'summary':
             import Summary
             text = Summary.build_report(self.state.db,None)
-            print text
         elif action == "report":
             try:
                 options_str_dict = dict( [ tuple(chunk.split('='))
@@ -674,7 +680,7 @@ class ArgHandler:
                     category = item[1]
                     tool_class = item[2]
                     options_class = item[3]
-                    Tool.cli_tool(self.state.db,name,category,
+                    Tool.cli_tool(self.state,name,category,
                                   tool_class,options_class,options_str_dict)
                     return
 
@@ -737,18 +743,18 @@ class NewNativeDbPrompter:
         while (True):
             response = choose.run()
             if response == gtk.RESPONSE_OK:
-                filename = choose.get_filename()
+                filename = unicode(choose.get_filename(),
+                                   sys.getfilesystemencoding())
                 if filename == None:
                     continue
                 if os.path.splitext(filename)[1] != ".grdb":
                     filename = filename + ".grdb"
                 choose.destroy()
-                print Config.TRANSACTIONS
                 self.state.db = GrampsDb.gramps_db_factory(const.app_gramps)(
                     Config.TRANSACTIONS)
-                self.vm.read_file(filename)
+                self.vm.db_loader.read_file(filename, const.app_gramps)
                 self.state.signal_change()
-                self.change_page(None, None)
+                #self.change_page(None, None)
                 # Add the file to the recent items
                 RecentFiles.recent_files(filename,const.app_gramps)
                 return True
