@@ -50,7 +50,7 @@
 #   time)
 #
 
-# $Id: BaseDoc.py 7063 2006-07-23 23:16:21Z dallingham $
+# $Id: BaseDoc.py 8000 2007-01-28 05:08:49Z dallingham $
 
 """
 Provides base interface to text based documents. Specific document
@@ -58,7 +58,7 @@ interfaces should be derived from the core classes.
 """
 
 __author__ = "Donald N. Allingham"
-__revision__ = "Revision:$Id: BaseDoc.py 7063 2006-07-23 23:16:21Z dallingham $"
+__revision__ = "Revision:$Id: BaseDoc.py 8000 2007-01-28 05:08:49Z dallingham $"
 
 #-------------------------------------------------------------------------
 #
@@ -67,6 +67,10 @@ __revision__ = "Revision:$Id: BaseDoc.py 7063 2006-07-23 23:16:21Z dallingham $"
 #-------------------------------------------------------------------------
 import os
 from math import pi, cos, sin
+from xml.sax.saxutils import escape
+
+def escxml(d):
+    return escape(d, { '"' : '&quot;' } )
 
 #-------------------------------------------------------------------------
 #
@@ -872,10 +876,10 @@ class StyleSheetList:
             if name == "default":
                 continue
             sheet = self.map[name]
-            xml_file.write('<sheet name="%s">\n' % name)
+            xml_file.write('<sheet name="%s">\n' % escxml(name))
             for p_name in sheet.get_names():
                 para = sheet.get_style(p_name)
-                xml_file.write('<style name="%s">\n' % p_name)
+                xml_file.write('<style name="%s">\n' % escxml(p_name))
                 font = para.get_font()
                 xml_file.write('<font face="%d" ' % font.get_type_face())
                 xml_file.write('size="%d" ' % font.get_size())
@@ -890,6 +894,7 @@ class StyleSheetList:
                 tmargin = float(para.get_top_margin())
                 bmargin = float(para.get_bottom_margin())
                 padding = float(para.get_padding())
+                xml_file.write('description="%s" ' % escxml(para.get_description()))
                 xml_file.write('rmargin="%s" ' % Utils.gformat(rmargin))
                 xml_file.write('lmargin="%s" ' % Utils.gformat(lmargin))
                 xml_file.write('first="%s" ' % Utils.gformat(findent))
@@ -1027,6 +1032,9 @@ class SheetParser(handler.ContentHandler):
             self.f.set_underline(int(attrs['underline']))
             self.f.set_color(cnv2color(attrs['color']))
         elif tag == "para":
+            if attrs.has_key('description'):
+                self.p.set_description(attrs['description'])
+            self.p.set_right_margin(Utils.gfloat(attrs['rmargin']))
             self.p.set_right_margin(Utils.gfloat(attrs['rmargin']))
             self.p.set_left_margin(Utils.gfloat(attrs['lmargin']))
             self.p.set_first_indent(Utils.gfloat(attrs['first']))
