@@ -17,7 +17,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _RelationView.py 7670 2006-11-23 05:01:18Z dallingham $
+# $Id: _RelationView.py 7984 2007-01-26 00:15:21Z dallingham $
+
+"""
+Relationship View
+"""
+
+__author__ = "Don Allingham"
+__revision__ = "$Revision: 7984 $"
 
 #-------------------------------------------------------------------------
 #
@@ -29,7 +36,7 @@ import cgi
 
 try:
     set()
-except:
+except NameError:
     from sets import Set as set
     
 #-------------------------------------------------------------------------
@@ -110,18 +117,26 @@ class RelationshipView(PageView.PersonNavView):
         PageView.PersonNavView.__init__(
             self, _('Relationships'), dbstate, uistate)
         
+        self.func_list = {
+            '<CONTROL>J' : self.jump,
+            }
+
         dbstate.connect('database-changed', self.change_db)
         self.show_siblings = Config.get(Config.FAMILY_SIBLINGS)
         self.show_details = Config.get(Config.FAMILY_DETAILS)
         self.connect_to_db(dbstate.db)
         self.redrawing = False
         self.use_shade = Config.get(Config.RELATION_SHADE)
+        self.toolbar_visible = Config.get(Config.TOOLBAR_ON)
         self.color = gtk.TextView().style.white
         self.child = None
         Config.client.notify_add("/apps/gramps/preferences/relation-shade",
                                  self.shade_update)
         Config.client.notify_add("/apps/gramps/interface/editbutton",
                                  self.config_update)
+        Config.client.notify_add("/apps/gramps/interface/toolbar-on",
+                                 self.shade_update)
+	self.reorder_sensitive = False
 
     def set_active(self):
         PageView.PersonNavView.set_active(self)
@@ -135,6 +150,7 @@ class RelationshipView(PageView.PersonNavView):
         
     def shade_update(self, client, cnxn_id, entry, data):
         self.use_shade = Config.get(Config.RELATION_SHADE)
+        self.toolbar_visible = Config.get(Config.TOOLBAR_ON)
         self.uistate.modify_statusbar(self.dbstate)
         self.redraw()
 
@@ -250,6 +266,7 @@ class RelationshipView(PageView.PersonNavView):
               </placeholder>
             </menu>
             <menu action="EditMenu">
+              <menuitem action="Edit"/>
               <menuitem action="AddParents"/>
               <menuitem action="AddSpouse"/>
               <menuitem action="ShareFamily"/>
@@ -274,6 +291,7 @@ class RelationshipView(PageView.PersonNavView):
               <toolitem action="HomePerson"/>
             </placeholder>
             <placeholder name="CommonEdit">
+              <toolitem action="Edit"/>
               <toolitem action="AddParents"/>
               <toolitem action="ShareFamily"/>
               <toolitem action="AddSpouse"/>
@@ -299,11 +317,13 @@ class RelationshipView(PageView.PersonNavView):
 
         self.family_action = gtk.ActionGroup(self.title + '/Family')
         self.family_action.add_actions([
-            ('AddSpouse', 'gramps-spouse', _('Add Spouse'), None ,
+            ('Edit', gtk.STOCK_EDIT, _('Edit'), None ,
+	     _("Edits the active person"), self.edit_active),
+            ('AddSpouse', 'gramps-spouse', _('Spouse'), None ,
 	     _("Adds a new relationship"), self.add_spouse),
-            ('AddParents', 'gramps-parents', _('Add Parents'), None ,
+            ('AddParents', 'gramps-parents', _('Add'), None ,
 	     _("Adds a new set of parents"), self.add_parents),
-            ('ShareFamily', 'gramps-sharefamily', _('Share Parents'), None ,
+            ('ShareFamily', 'gramps-sharefamily', _('Share'), None ,
 	     _("Adds an existing set of parents"), self.select_parents),
             ])
 
@@ -317,21 +337,25 @@ class RelationshipView(PageView.PersonNavView):
                                None, None, self.siblings_toggle, 
                                self.show_siblings)
 
+	self.order_action.set_sensitive(self.reorder_sensitive)
+
     def siblings_toggle(self, obj):
         self.show_siblings = obj.get_active()
         self.change_person(self.dbstate.active.handle)
-        Config.set(Config.FAMILY_SIBLINGS,self.show_siblings)
+        Config.set(Config.FAMILY_SIBLINGS, self.show_siblings)
 
     def details_toggle(self, obj):
         self.show_details = obj.get_active()
         self.change_person(self.dbstate.active.handle)
-        Config.set(Config.FAMILY_DETAILS,self.show_details)
+        Config.set(Config.FAMILY_DETAILS, self.show_details)
 
     def change_db(self, db):
         self.connect_to_db(db)
         if self.child:
             for old_child in self.vbox.get_children():
                 self.vbox.remove(old_child)
+            for old_child in self.header.get_children():
+                self.header.remove(old_child)
             self.child = None
         self.dbstate.db.connect('family-update', self.redraw)
         self.dbstate.db.connect('family-add', self.redraw)
@@ -345,10 +369,10 @@ class RelationshipView(PageView.PersonNavView):
 
     def get_name(self, handle, use_gender=False):
         if handle:
-            p = self.dbstate.db.get_person_from_handle(handle)
-            name = NameDisplay.displayer.display(p)
+            person = self.dbstate.db.get_person_from_handle(handle)
+            name = NameDisplay.displayer.display(person)
             if use_gender:
-                gender = _GenderCode[p.gender]
+                gender = _GenderCode[person.gender]
             else:
                 gender = ""
             return (name, gender)
@@ -363,17 +387,31 @@ class RelationshipView(PageView.PersonNavView):
             self.change_person(None)
         
     def change_person(self, obj):
+        try:
+            return self._change_person(obj)
+        except AttributeError, msg:
+            from QuestionDialog import RunDatabaseRepair
+            RunDatabaseRepair(msg)
+            return True
+
+    def _change_person(self, obj):
         if self.redrawing:
             return False
         self.redrawing = True
+        self.tooltips = gtk.Tooltips()
 
         for old_child in self.vbox.get_children():
             self.vbox.remove(old_child)
+        for old_child in self.header.get_children():
+            self.header.remove(old_child)
 
         person = self.dbstate.db.get_person_from_handle(obj)
         if not person:
+            self.family_action.set_sensitive(False)
+            self.order_action.set_sensitive(False)
             self.redrawing = False
             return
+        self.family_action.set_sensitive(True)
 
         self.write_title(person)
 
@@ -382,7 +420,7 @@ class RelationshipView(PageView.PersonNavView):
 
         family_handle_list = person.get_parent_family_handle_list()
 
-	sensitive = len(family_handle_list)> 1
+        self.reorder_sensitive = len(family_handle_list)> 1
 
         if family_handle_list:
             for family_handle in family_handle_list:
@@ -394,8 +432,8 @@ class RelationshipView(PageView.PersonNavView):
                 
         family_handle_list = person.get_family_handle_list()
 	
-	if not sensitive:
-	    sensitive = len(family_handle_list)> 1
+        if not self.reorder_sensitive:
+            self.reorder_sensitive = len(family_handle_list)> 1
 
         if family_handle_list:
             for family_handle in family_handle_list:
@@ -444,7 +482,7 @@ class RelationshipView(PageView.PersonNavView):
         self.redrawing = False
         self.uistate.modify_statusbar(self.dbstate)
 
-	self.order_action.set_sensitive(sensitive)
+        self.order_action.set_sensitive(self.reorder_sensitive)
 
         return True
 
@@ -453,7 +491,7 @@ class RelationshipView(PageView.PersonNavView):
         for old_child in self.header.get_children():
             self.header.remove(old_child)
 
-        table = gtk.Table(2,3)
+        table = gtk.Table(2, 3)
         table.set_col_spacings(12)
         table.set_row_spacings(6)
         
@@ -463,7 +501,8 @@ class RelationshipView(PageView.PersonNavView):
         text = fmt % cgi.escape(name)
         label = GrampsWidgets.DualMarkupLabel(text, _GenderCode[person.gender])
         if Config.get(Config.RELEDITBTN):
-            button = GrampsWidgets.IconButton(self.edit_button_press,person.handle)
+            button = GrampsWidgets.IconButton(self.edit_button_press, person.handle)
+            self.tooltips.set_tip(button, _('Edit %s') % name)
         else:
             button = None
         hbox = GrampsWidgets.LinkBox(label, button)
@@ -487,22 +526,22 @@ class RelationshipView(PageView.PersonNavView):
                       2, 3, 0, 1, yoptions=0)
 
         # Birth event.
-        birth = ReportUtils.get_birth_or_fallback(self.dbstate.db,person)
+        birth = ReportUtils.get_birth_or_fallback(self.dbstate.db, person)
         if birth:
             birth_title = birth.get_type()
         else:
-            birth_title = None
+            birth_title = _("Birth")
 
         subtbl.attach(GrampsWidgets.BasicLabel("%s:" % birth_title),
                       1, 2, 1, 2, xoptions=gtk.FILL, yoptions=0)
         subtbl.attach(GrampsWidgets.BasicLabel(self.format_event(birth)),
                       2, 3, 1, 2, yoptions=0)
 
-        death = ReportUtils.get_death_or_fallback(self.dbstate.db,person)
+        death = ReportUtils.get_death_or_fallback(self.dbstate.db, person)
         if death:
             death_title = death.get_type()
         else:
-            death_title = None
+            death_title = _("Death")
 
         subtbl.attach(GrampsWidgets.BasicLabel("%s:" % death_title),
                       1, 2, 2, 3, xoptions=gtk.FILL, yoptions=0)
@@ -522,10 +561,10 @@ class RelationshipView(PageView.PersonNavView):
                 image = gtk.Image()
                 image.set_from_pixbuf(pixbuf)
                 image.show()
-                mbox.pack_end(image,False)
+                mbox.pack_end(image, False)
 
         mbox.show_all()
-        self.header.pack_start(mbox,False)
+        self.header.pack_start(mbox, False)
 
     def write_person_event(self, ename, event):
         if event:
@@ -610,24 +649,45 @@ class RelationshipView(PageView.PersonNavView):
         if is_parent:
             call_fcn = self.add_parent_family
             del_fcn = self.delete_parent_family
+            add_msg = _('Add parents')
+            sel_msg = _('Select existing parents')
+            edit_msg = _('Edit parents')
+            del_msg = _('Remove parents')
         else:
+            add_msg = _('Add spouse')
+            sel_msg = _('Select spouse')
+            edit_msg = _('Edit family')
+            del_msg = _('Remove from family')
             call_fcn = self.add_family
             del_fcn = self.delete_family
-            
-#        add = GrampsWidgets.IconButton(call_fcn, None, gtk.STOCK_ADD)
-#        hbox.pack_start(add, False)
+        
+        if not self.toolbar_visible and not self.dbstate.db.readonly:
+            # Show edit-Buttons if toolbar is not visible
+            if self.reorder_sensitive:
+                add = GrampsWidgets.IconButton(self.reorder, None, 
+					       gtk.STOCK_SORT_ASCENDING)
+                self.tooltips.set_tip(add, _('Reorder families'))
+                hbox.pack_start(add, False)
 
-#         if is_parent:
-#             add = GrampsWidgets.IconButton(self.select_family, None, gtk.STOCK_INDEX)
-#             hbox.pack_start(add, False)
+            add = GrampsWidgets.IconButton(call_fcn, None, gtk.STOCK_ADD)
+            self.tooltips.set_tip(add, add_msg)
+            hbox.pack_start(add, False)
+    
+            if is_parent:
+                add = GrampsWidgets.IconButton(self.select_family, None, gtk.STOCK_INDEX)
+                self.tooltips.set_tip(add, sel_msg)
+                hbox.pack_start(add, False)
 
         if family:
             edit = GrampsWidgets.IconButton(self.edit_family, family.handle, 
                                             gtk.STOCK_EDIT)
+            self.tooltips.set_tip(edit, edit_msg)
             hbox.pack_start(edit, False)
-            delete = GrampsWidgets.IconButton(del_fcn, family.handle, 
-                                              gtk.STOCK_REMOVE)
-            hbox.pack_start(delete, False)
+            if not self.dbstate.db.readonly:
+                delete = GrampsWidgets.IconButton(del_fcn, family.handle, 
+                                                  gtk.STOCK_REMOVE)
+                self.tooltips.set_tip(delete, del_msg)
+                hbox.pack_start(delete, False)
         self.attach.attach(hbox, _BTN_START, _BTN_STOP, self.row, self.row+1)
         self.row += 1
         
@@ -678,9 +738,9 @@ class RelationshipView(PageView.PersonNavView):
             format = "%s"
 
         label = GrampsWidgets.MarkupLabel(format % cgi.escape(title))
-        label.set_alignment(0,0)
+        label.set_alignment(0, 0)
         if Config.get(Config.RELEDITBTN):
-            label.set_padding(0,5)
+            label.set_padding(0, 5)
         self.attach.attach(label, _PLABEL_START, _PLABEL_STOP, self.row, 
                            self.row+1, xoptions=gtk.FILL|gtk.SHRINK,
                            yoptions=gtk.FILL|gtk.SHRINK)
@@ -688,12 +748,14 @@ class RelationshipView(PageView.PersonNavView):
         vbox = gtk.VBox()
         
         if handle:
-            link_label = GrampsWidgets.LinkLabel(self.get_name(handle, True), 
+            name = self.get_name(handle, True)
+            link_label = GrampsWidgets.LinkLabel(name, 
                                                  self.button_press, handle)
             if self.use_shade:
                 link_label.modify_bg(gtk.STATE_NORMAL, self.color)
             if Config.get(Config.RELEDITBTN):
                 button = GrampsWidgets.IconButton(self.edit_button_press, handle)
+                self.tooltips.set_tip(button, _('Edit %s') % name[0])
             else:
                 button = None
             vbox.pack_start(GrampsWidgets.LinkBox(link_label, button))
@@ -726,7 +788,7 @@ class RelationshipView(PageView.PersonNavView):
 
         lbl = GrampsWidgets.MarkupLabel(format % cgi.escape(title))
         if Config.get(Config.RELEDITBTN):
-            lbl.set_padding(0,5)
+            lbl.set_padding(0, 5)
         return lbl
 
     def write_child(self, vbox, handle, index):
@@ -766,17 +828,17 @@ class RelationshipView(PageView.PersonNavView):
 
         birth = ReportUtils.get_birth_or_fallback(self.dbstate.db, child)
         if birth and birth.get_type != RelLib.EventType.BIRTH:
-            bdate  = "<i>%s</i>" % DateHandler.get_date(birth)
+            bdate  = "<i>%s</i>" % cgi.escape(DateHandler.get_date(birth))
         elif birth:
-            bdate  = DateHandler.get_date(birth)
+            bdate  = cgi.escape(DateHandler.get_date(birth))
         else:
             bdate = ""
 
         death = ReportUtils.get_death_or_fallback(self.dbstate.db, child)
         if death and death.get_type != RelLib.EventType.DEATH:
-            ddate  = "<i>%s</i>" % DateHandler.get_date(death)
+            ddate  = "<i>%s</i>" % cgi.escape(DateHandler.get_date(death))
         elif death:
-            ddate  = DateHandler.get_date(death)
+            ddate  = cgi.escape(DateHandler.get_date(death))
         else:
             ddate = ""
 
@@ -793,39 +855,39 @@ class RelationshipView(PageView.PersonNavView):
     def button_press(self, obj, event, handle):
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
             self.dbstate.change_active_handle(handle)
-	elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
-	    myMenu = gtk.Menu()
-	    myMenu.append(self.build_menu_item(handle))
-	    myMenu.popup(None, None, None, event.button, event.time)
+        elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+            myMenu = gtk.Menu()
+            myMenu.append(self.build_menu_item(handle))
+            myMenu.popup(None, None, None, event.button, event.time)
 
     def build_menu_item(self, handle):
-	person = self.dbstate.db.get_person_from_handle(handle)
-	name = NameDisplay.displayer.display(person)
+        person = self.dbstate.db.get_person_from_handle(handle)
+        name = NameDisplay.displayer.display(person)
 
-	item = gtk.ImageMenuItem(None)
-	image = gtk.image_new_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU)
-	image.show()
-	label = gtk.Label(_("Edit %s") % name)
-	label.show()
-	label.set_alignment(0,0)
+        item = gtk.ImageMenuItem(None)
+        image = gtk.image_new_from_stock(gtk.STOCK_EDIT, gtk.ICON_SIZE_MENU)
+        image.show()
+        label = gtk.Label(_("Edit %s") % name)
+        label.show()
+        label.set_alignment(0, 0)
 
-	item.set_image(image)
-	item.add(label)
+        item.set_image(image)
+        item.add(label)
 
-	item.connect('activate',self.edit_menu, handle)
-	item.show()
-	return item
+        item.connect('activate', self.edit_menu, handle)
+        item.show()
+        return item
 
     def edit_menu(self, obj, handle):
-	from Editors import EditPerson
-	person = self.dbstate.db.get_person_from_handle(handle)
-	try:
-	    EditPerson(self.dbstate, self.uistate, [], person)
-	except Errors.WindowActiveError:
-	    pass
+        from Editors import EditPerson
+        person = self.dbstate.db.get_person_from_handle(handle)
+        try:
+            EditPerson(self.dbstate, self.uistate, [], person)
+        except Errors.WindowActiveError:
+            pass
 
     def write_relationship(self, box, family):
-        msg = _('Relationship type: %s') % str(family.get_relationship())
+        msg = _('Relationship type: %s') % cgi.escape(str(family.get_relationship()))
         box.add(GrampsWidgets.MarkupLabel(msg))
 
     def place_name(self, handle):
@@ -909,7 +971,7 @@ class RelationshipView(PageView.PersonNavView):
                 eventbox.modify_bg(gtk.STATE_NORMAL, self.color)
             vbox = gtk.VBox()
             label_cell = self.build_label_cell(_('Children'))
-            label_cell.set_alignment(0,0)
+            label_cell.set_alignment(0, 0)
             self.attach.attach(
                 label_cell, _CLABEL_START, _CLABEL_STOP, self.row, 
                 self.row+1, xoptions=gtk.FILL|gtk.SHRINK,
@@ -929,12 +991,7 @@ class RelationshipView(PageView.PersonNavView):
 
     def edit_button_press(self, obj, event, handle):
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
-            from Editors import EditPerson
-            person = self.dbstate.db.get_person_from_handle(handle)
-            try:
-                EditPerson(self.dbstate, self.uistate, [], person)
-            except Errors.WindowActiveError:
-                pass
+            self.edit_person(obj, handle)
         
     def edit_person(self, obj, handle):
         from Editors import EditPerson
@@ -984,6 +1041,10 @@ class RelationshipView(PageView.PersonNavView):
         except Errors.WindowActiveError:
             pass
 
+    def edit_active(self, obj):
+        phandle = self.dbstate.get_active_person().handle
+        self.edit_person(obj, phandle)
+
     def select_family(self, obj, event, handle):
         if event.type == gtk.gdk.BUTTON_PRESS and event.button == 1:
             from Selectors import selector_factory
@@ -1069,11 +1130,11 @@ class RelationshipView(PageView.PersonNavView):
     def change_to(self, obj, handle):
         self.dbstate.change_active_handle(handle)
 
-    def reorder(self,obj):
+    def reorder(self, obj, dumm1=None, dummy2=None):
         if self.dbstate.active:
             try:
-		import Reorder
-		Reorder.Reorder(self.dbstate, self.uistate, [],
-				self.dbstate.active.handle)
+                import Reorder
+                Reorder.Reorder(self.dbstate, self.uistate, [],
+                                self.dbstate.active.handle)
             except Errors.WindowActiveError:
                 pass
