@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: DateEdit.py 6992 2006-07-04 23:38:51Z dallingham $
+# $Id: DateEdit.py 7951 2007-01-22 03:56:23Z dallingham $
 
 """
 Date editing module for GRAMPS. 
@@ -34,7 +34,7 @@ unambiguously built using UI controls such as menus and spin buttons.
 """
 
 __author__ = "Donald N. Allingham"
-__version__ = "$Revision: 6992 $"
+__version__ = "$Revision: 7951 $"
 
 #-------------------------------------------------------------------------
 #
@@ -68,6 +68,7 @@ import DateHandler
 import const
 import GrampsDisplay
 import ManagedWindow
+from Errors import ValidationError
 
 #-------------------------------------------------------------------------
 #
@@ -118,48 +119,37 @@ class DateEdit:
         self.text_obj = text_obj
         self.button_obj = button_obj
 
+        image = gtk.Image()
+        image.set_from_stock('gramps-date', gtk.ICON_SIZE_BUTTON)
+        self.button_obj.set_image(image)
+        self.button_obj.set_relief(gtk.RELIEF_NORMAL)
         self.pixmap_obj = button_obj.get_child()
         
-        self.text_obj.connect('focus-out-event',self.parse_and_check)
+        self.text_obj.connect('validate',self.validate)
+        self.text_obj.connect('content-changed',self.set_date)
         self.button_obj.connect('clicked',self.invoke_date_editor)
         
-        self.text = unicode(self.text_obj.get_text())
-        self.check()
-
-    def check(self):
-        """
-        Check current date object and display LED indicating the validity.
-        """
-        if self.date_obj.get_modifier() == Date.MOD_TEXTONLY:
-            self.pixmap_obj.set_from_pixbuf(
-                self.pixmap_obj.render_icon(gtk.STOCK_DIALOG_ERROR,
-                                            gtk.ICON_SIZE_MENU))
-        elif self.date_obj.is_regular():
-            self.pixmap_obj.set_from_pixbuf(
-                self.pixmap_obj.render_icon(gtk.STOCK_YES,
-                                            gtk.ICON_SIZE_MENU))
-        else:
-            self.pixmap_obj.set_from_pixbuf(
-                self.pixmap_obj.render_icon(gtk.STOCK_DIALOG_WARNING,
-                                            gtk.ICON_SIZE_MENU))
-
+        self.text_obj.set_text(DateHandler.displayer.display(self.date_obj))
+        self.text_obj.validate()
         
-    def parse_and_check(self,*obj):
+    def set_date(self, widget):
         """
-        Called with the text box loses focus. Parses the text and calls 
-        the check() method ONLY if the text has changed.
+        Parse date from text entry to date object
         """
-
-        text = unicode(self.text_obj.get_text())
-        if text != self.text:
-            self.text = text
-            self.date_obj.copy(DateHandler.parser.parse(text))
-            self.text_obj.set_text(DateHandler.displayer.display(self.date_obj))
-            self.check()
+        date = DateHandler.parser.parse(unicode(self.text_obj.get_text()))
+        self.date_obj.copy(date)
+        
+    def validate(self, widget, data):
+        """
+        Validate current date in text entry
+        """
+        # if text could not be parsed it is assumed invalid
+        if self.date_obj.get_modifier() == Date.MOD_TEXTONLY:
+            return ValidationError(_('Bad Date'))
 
     def invoke_date_editor(self,obj):
         """
-        Invokes Date Editor dialog when the user clicks the LED button.
+        Invokes Date Editor dialog when the user clicks the Calendar button.
         If date was in fact built, sets the date_obj to the newly built
         date.
         """
@@ -169,12 +159,13 @@ class DateEdit:
 
     def update_after_editor(self,date_obj):
         """
-        Update text field and LED button to reflect the given date instance.
+        Update text entry and validate it
         """
         if date_obj:
-            self.date_obj.copy(date_obj)
-            self.text_obj.set_text(DateHandler.displayer.display(self.date_obj))
-            self.check()
+            # first we set the text entry, that emits 'content-changed'
+            # signal thus the date object gets updated too
+            self.text_obj.set_text(DateHandler.displayer.display(date_obj))
+            self.text_obj.validate()
         
 #-------------------------------------------------------------------------
 #
