@@ -30,7 +30,8 @@ Handling of loading new/existing databases.
 #
 #-------------------------------------------------------------------------
 import os
-from bsddb.db import DBAccessError, DBRunRecoveryError, DBPageNotFoundError
+import sys
+from bsddb.db import DBAccessError, DBRunRecoveryError, DBPageNotFoundError, DBInvalidArgError
 from gettext import gettext as _
 import logging
 log = logging.getLogger(".")
@@ -98,7 +99,8 @@ class DbLoader:
         choose.set_current_folder(get_default_dir())
         response = choose.run()
         if response == gtk.RESPONSE_OK:
-            filename = choose.get_filename()
+            filename = unicode(choose.get_filename(),
+                               sys.getfilesystemencoding())
             if self.check_errors(filename):
                 return ('','')
 
@@ -116,6 +118,13 @@ class DbLoader:
                 except:
                     return ('','')
                 return (filename,filetype)
+            elif filetype in [const.app_gramps_package,const.app_geneweb]:
+                QuestionDialog.ErrorDialog(
+                    _("Could not open file: %s") % filename, 
+                    _('Files of type "%s" cannot be opened directly.\n\n'
+                      'Please create a new GRAMPS database and import '
+                      'the file.') % filetype)
+                return ('','')
             else:
                 QuestionDialog.ErrorDialog(
                     _("Could not open file: %s") % filename, 
@@ -147,7 +156,8 @@ class DbLoader:
         while (True):
             response = choose.run()
             if response == gtk.RESPONSE_OK:
-                filename = choose.get_filename()
+                filename = unicode(choose.get_filename(),
+                                   sys.getfilesystemencoding())
                 if self.check_errors(filename):
                     return ('','')
 
@@ -209,10 +219,15 @@ class DbLoader:
 
         response = choose.run()
         if response == gtk.RESPONSE_OK:
-            filename = choose.get_filename()
+            filename = unicode(choose.get_filename(),
+                               sys.getfilesystemencoding())
             if self.check_errors(filename):
                 return ('','')
 
+            # Do not allow saving as into the currently open file
+            if filename == self.dbstate.db.full_name:
+                return ('','')
+
             filetype = type_selector.get_value()
             if filetype == 'auto':
                 try:
@@ -296,10 +311,15 @@ class DbLoader:
         choose.set_current_folder(default_dir)
         response = choose.run()
         if response == gtk.RESPONSE_OK:
-            filename = choose.get_filename()
+            filename = unicode(choose.get_filename(),
+                               sys.getfilesystemencoding())
             if self.check_errors(filename):
                 return False
 
+            # Do not allow importing from the currently open file
+            if filename == self.dbstate.db.full_name:
+                return False
+            
             filetype = type_selector.get_value()
             if filetype == 'auto':
                 try:
@@ -407,7 +427,15 @@ class DbLoader:
         else:
             mode = 'w'
 
-        dbclass = GrampsDb.gramps_db_factory(db_type = filetype)
+        try:
+            dbclass = GrampsDb.gramps_db_factory(db_type = filetype)
+        except GrampsDb.GrampsDbException, msg:
+            QuestionDialog.ErrorDialog(
+                _("Could not open file: %s") % filename, 
+                _("This may be caused by an improper installation of GRAMPS.") +
+                "\n" + str(msg))
+            return
+                
         
         self.dbstate.change_database(dbclass(Config.get(Config.TRANSACTIONS)))
         self.dbstate.db.disable_signals()
@@ -432,7 +460,7 @@ class DbLoader:
                       "problem persists, create a new database, import "
                       "from a backup database, and report the problem to "
                       "gramps-bugs@lists.sourceforge.net."))
-        except (DBAccessError, DBPageNotFoundError), msg:
+        except (DBAccessError, DBPageNotFoundError,DBInvalidArgError), msg:
                 QuestionDialog.ErrorDialog(
                     _("Could not open file: %s") % filename,
                     str(msg[1]))
