@@ -18,14 +18,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _PeopleModel.py 7669 2006-11-22 22:30:47Z loshawlos $
+# $Id: _PeopleModel.py 7946 2007-01-21 05:17:06Z dallingham $
 
 """
 TreeModel for the GRAMPS Person tree.
 """
 
 __author__ = "Donald N. Allingham"
-__revision__ = "$Revision: 7669 $"
+__revision__ = "$Revision: 7946 $"
 
 #-------------------------------------------------------------------------
 #
@@ -65,9 +65,11 @@ import DateHandler
 import ToolTips
 import GrampsLocale
 import Config
-from Filters import SearchFilter
+from Filters import SearchFilter, ExactSearchFilter
 from Lru import LRU
 
+_CACHE_SIZE = 250
+
 #-------------------------------------------------------------------------
 #
 # python 2.3 has a bug in the unicode sorting using locale.strcoll. Seems
@@ -138,6 +140,7 @@ class PeopleModel(gtk.GenericTreeModel):
         gtk.GenericTreeModel.__init__(self)
 
         self.db = db
+	self.in_build = False
 
         Config.client.notify_add("/apps/gramps/preferences/todo-color",
                                  self.update_todo)
@@ -153,14 +156,12 @@ class PeopleModel(gtk.GenericTreeModel):
         self.sortnames = {}
         self.marker_color_column = 10
         self.tooltip_column = 11
-        self.prev_handle = None
-        self.prev_data = None
         self.temp_top_path2iter = []
         self.iter2path = {}
         self.path2iter = {}
         self.sname_sub = {}
-        
-        if filter_info:
+
+        if filter_info and filter_info != (1, (0, u'', False)):
             if filter_info[0] == PeopleModel.GENERIC:
                 data_filter = filter_info[1]
                 self._build_data = self._build_filter_sub
@@ -169,7 +170,12 @@ class PeopleModel(gtk.GenericTreeModel):
                 text = filter_info[1][1]
                 inv = filter_info[1][2]
                 func = lambda x: self.on_get_value(x, col) or u""
-                data_filter = SearchFilter(func, text, inv)
+
+                if col == self._GENDER_COL:
+                    data_filter = ExactSearchFilter(func, text, inv)
+                else:
+                    data_filter = SearchFilter(func, text, inv)
+
                 self._build_data = self._build_search_sub
             else:
                 data_filter = filter_info[1]
@@ -177,6 +183,7 @@ class PeopleModel(gtk.GenericTreeModel):
         else:
             self._build_data = self._build_search_sub
             data_filter = None
+        self.current_filter = data_filter
         self.rebuild_data(data_filter, skip)
 
     def update_todo(self,client,cnxn_id,entry,data):
@@ -194,6 +201,7 @@ class PeopleModel(gtk.GenericTreeModel):
         """
         self.calculate_data(data_filter, skip)
         self.assign_data()
+        self.current_filter = data_filter
         
     def _build_search_sub(self,dfilter, skip):
         self.sortnames = {}
@@ -243,8 +251,12 @@ class PeopleModel(gtk.GenericTreeModel):
         """
         Calculates the new path to node values for the model.
         """
+	self.in_build  = True
+        self.lru_data  = LRU(_CACHE_SIZE)
+        self.lru_name  = LRU(_CACHE_SIZE)
+        self.lru_bdate = LRU(_CACHE_SIZE)
+        self.lru_ddate = LRU(_CACHE_SIZE)
 
-        self.lru = LRU(500)
         if dfilter:
             self.dfilter = dfilter
         self.temp_iter2path = {}
@@ -259,13 +271,15 @@ class PeopleModel(gtk.GenericTreeModel):
         self.temp_top_path2iter = locale_sort(self.temp_sname_sub.keys())
         for name in self.temp_top_path2iter:
             self.build_sub_entry(name)
+	self.in_build  = False
 
     def clear_cache(self):
-        self.lru = LRU(500)
-        self.prev_handle = None
-        
+        self.lru_data  = LRU(_CACHE_SIZE)
+        self.lru_name  = LRU(_CACHE_SIZE)
+        self.lru_bdate = LRU(_CACHE_SIZE)
+        self.lru_ddate = LRU(_CACHE_SIZE)
+
     def build_sub_entry(self, name):
-        self.prev_handle = None
         slist = [ (locale.strxfrm(self.sortnames[x]), x) \
                   for x in self.temp_sname_sub[name] ]
         slist.sort()
@@ -282,6 +296,11 @@ class PeopleModel(gtk.GenericTreeModel):
         self.iter2path = self.temp_iter2path
         self.path2iter = self.temp_path2iter
         self.sname_sub = self.temp_sname_sub
+        self.top_iter2path = {}
+        i = 0
+        for item in self.top_path2iter:
+            self.top_iter2path[item] = i
+            i+=1
 
     def on_get_flags(self):
         '''returns the GtkTreeModelFlags for this particular type of model'''
@@ -294,10 +313,10 @@ class PeopleModel(gtk.GenericTreeModel):
         '''returns the tree path (a tuple of indices at the various
         levels) for a particular node.'''
         try:
-            return (self.top_path2iter.index(node), )
+            return (self.top_iter2path[node], )
         except:
             (surname, index) = self.iter2path[node]
-            return (self.top_path2iter.index(surname), index)
+            return (self.top_iter2path[surname], index)
 
     def is_visable(self, handle):
         return self.iter2path.has_key(handle)
@@ -332,18 +351,21 @@ class PeopleModel(gtk.GenericTreeModel):
             # return values for 'data' row, calling a function
             # according to column_defs table
             try:
-                if node != self.prev_handle:
-                    self.prev_data = self.db.get_raw_person_data(str(node))
-                    self.prev_handle = node
+                try:
+                    data = self.lru_data[node]
+                except:
+                    data = self.db.get_raw_person_data(str(node))
+		    if not self.in_build:
+			self.lru_data[node] = data
                 return PeopleModel.COLUMN_DEFS[col][PeopleModel.COLUMN_DEF_LIST](self,
-                                                         self.prev_data, node)
+                                                                                 data, node)
             except:
                 return None
 
     def on_iter_next(self,  node):
         '''returns the next node at this level of the tree'''
         try:
-            path = self.top_path2iter.index(node)
+            path = self.top_iter2path[node]
             if path+1 == len(self.top_path2iter):
                 return None
             return self.top_path2iter[path+1]
@@ -415,11 +437,12 @@ class PeopleModel(gtk.GenericTreeModel):
         return spouses_names
 
     def column_name(self, data, node):
-        try:
-            name = self.lru[node]
-        except:
-            name = NameDisplay.displayer.raw_sorted_name(data[PeopleModel._NAME_COL])
-            self.lru[node] = name
+	try:
+	    name = self.lru_name[node]
+	except:
+	    name = NameDisplay.displayer.raw_sorted_name(data[PeopleModel._NAME_COL])
+	    if not self.in_build:
+		self.lru_name[node] = name
         return name
 
     def column_id(self, data, node):
@@ -435,6 +458,15 @@ class PeopleModel(gtk.GenericTreeModel):
         return PeopleModel._GENDER[data[PeopleModel._GENDER_COL]]
 
     def column_birth_day(self, data, node):
+	try:
+	    value = self.lru_bdate[node]
+	except:
+	    value = self._get_birth_data(data,node)
+	    if not self.in_build:
+		self.lru_bdate[node] = value
+        return value
+
+    def _get_birth_data(self, data, node):
         index = data[PeopleModel._BIRTH_COL]
         if index != -1:
             try:
@@ -462,6 +494,15 @@ class PeopleModel(gtk.GenericTreeModel):
         return u""
 
     def column_death_day(self, data, node):
+	try:
+	    value = self.lru_ddate[node]
+	except:
+	    value = self._get_death_data(data,node)
+	    if not self.in_build:
+		self.lru_ddate[node] = value
+        return value
+
+    def _get_death_data(self, data, node):
         index = data[PeopleModel._DEATH_COL]
         if index != -1:
             try:
@@ -569,7 +610,7 @@ class PeopleModel(gtk.GenericTreeModel):
             if data[PeopleModel._MARKER_COL]:
                 if data[PeopleModel._MARKER_COL][0] == MarkerType.COMPLETE:
                     return self.complete_color
-                if data[PeopleModel._MARKER_COL][0] == MarkerType.TODO:
+                if data[PeopleModel._MARKER_COL][0] == MarkerType.TODO_TYPE:
                     return self.todo_color
                 if data[PeopleModel._MARKER_COL][0] == MarkerType.CUSTOM:
                     return self.custom_color
