@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _EditFamily.py 7710 2006-11-26 21:50:21Z dallingham $
+# $Id: _EditFamily.py 7915 2007-01-16 07:12:10Z loshawlos $
 
 #-------------------------------------------------------------------------
 #
@@ -118,11 +118,11 @@ class ChildEmbedList(EmbeddedList):
 
     def get_popup_menu_items(self):
         return [
-            (True, gtk.STOCK_ADD, self.add_button_clicked),
-            (False, _('Share'), self.edit_button_clicked),
-            (True,  _('Edit relationship'), self.edit_button_clicked),
-            (True,  _('Edit child'), self.edit_child_button_clicked),
-            (True, gtk.STOCK_REMOVE, self.del_button_clicked),
+            (True, True, gtk.STOCK_ADD, self.add_button_clicked),
+            (True, False, _('Share'), self.edit_button_clicked),
+            (False,True,  _('Edit relationship'), self.edit_button_clicked),
+            (False,True,  _('Edit child'), self.edit_child_button_clicked),
+            (True, True, gtk.STOCK_REMOVE, self.del_button_clicked),
             ]
 
     def find_index(self,obj):
@@ -468,7 +468,6 @@ class EditFamily(EditPrimary):
 
         self.mbox    = self.top.get_widget('mbox')
         self.fbox    = self.top.get_widget('fbox')
-        self.window.show()
 
     def _connect_signals(self):
         self.define_ok_button(self.top.get_widget('ok'), self.save)
@@ -559,6 +558,7 @@ class EditFamily(EditPrimary):
             FamilyLdsEmbedList(self.dbstate,self.uistate,self.track,
                                self.obj.get_lds_ord_list()))
 
+        self._setup_notebook_tabs( notebook)
         notebook.show_all()
 
         self.hidden = (notebook, self.top.get_widget('info'))
