@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _EditPerson.py 7711 2006-11-26 22:05:35Z dallingham $
+# $Id: _EditPerson.py 7995 2007-01-27 18:28:29Z dallingham $
 
 """
 EditPerson Dialog. Provides the interface to allow the GRAMPS program
@@ -26,7 +26,7 @@ to edit information about a particular Person.
 """
 
 __author__ = "Don Allingham"
-__revision__ = "$Revision: 7711 $"
+__revision__ = "$Revision: 7995 $"
 
 #-------------------------------------------------------------------------
 #
@@ -59,6 +59,7 @@ import Mime
 import RelLib
 import GrampsWidgets
 import NameDisplay
+import Errors
 
 from GrampsDb import set_birth_death_index
 
@@ -67,7 +68,7 @@ from QuestionDialog import *
 from DisplayTabs import \
      PersonEventEmbedList,NameEmbedList,SourceEmbedList,AttrEmbedList,\
      AddrEmbedList,NoteTab,GalleryTab,WebEmbedList,PersonRefEmbedList, \
-     LdsEmbedList
+     LdsEmbedList,PersonBackRefList
     
 #-------------------------------------------------------------------------
 #
@@ -164,6 +165,26 @@ class EditPerson(EditPrimary):
         self.eventbox.connect('button-press-event',
                               self._image_button_press)
 
+        self._add_db_signal('family-rebuild', self.family_change)
+        self._add_db_signal('family-delete', self.family_change)
+        self._add_db_signal('family-update', self.family_change)
+        self._add_db_signal('family-add', self.family_change)
+
+    def family_change(self, handle_list):
+        flist = self.obj.get_family_handle_list() + self.obj.get_parent_family_handle_list()
+        for handle in handle_list:
+            if handle in flist:
+                self._update_families()
+                return
+
+    def _update_families(self):
+        phandle = self.obj.get_handle()
+        person = self.dbstate.db.get_person_from_handle(phandle)
+        self.obj.set_family_handle_list(person.get_family_handle_list())
+        self.obj.set_parent_family_handle_list(person.get_parent_family_handle_list())
+        self.person_ref_list.data = self.obj.get_person_ref_list()
+        self.person_ref_list.rebuild()
+
     def _setup_fields(self):
         """
         Connects the GrampsWidget objects to field in the interface. This
@@ -173,7 +194,8 @@ class EditPerson(EditPrimary):
 
         self.private = GrampsWidgets.PrivacyButton(
             self.top.get_widget('private'), 
-            self.obj)
+            self.obj, 
+            self.db.readonly)
 
         self.gender = GrampsWidgets.MonitoredMenu(
             self.top.get_widget('gender'), 
@@ -304,16 +326,20 @@ class EditPerson(EditPrimary):
             WebEmbedList(self.dbstate, self.uistate, self.track, 
                          self.obj.get_url_list()))
 
-        self.pref_list = self._add_tab(
-            notebook, 
-            PersonRefEmbedList(self.dbstate, self.uistate, self.track, 
-                               self.obj.get_person_ref_list()))
-
+        self.person_ref_list = PersonRefEmbedList(self.dbstate, self.uistate, self.track, 
+                                                  self.obj.get_person_ref_list())
+        self.pref_list = self._add_tab(notebook, self.person_ref_list)
         self.lds_list = self._add_tab(
             notebook, 
             LdsEmbedList(self.dbstate, self.uistate, self.track, 
                          self.obj.get_lds_ord_list()))
 
+        self.backref_tab = self._add_tab(
+            notebook,
+            PersonBackRefList(self.dbstate, self.uistate, self.track,
+                              self.db.find_backlink_handles(self.obj.handle)))
+
+        self._setup_notebook_tabs( notebook)
         notebook.show_all()
         self.top.get_widget('vbox').pack_start(notebook, True)
 
@@ -382,8 +408,12 @@ class EditPerson(EditPrimary):
                 object_handle = media_ref.get_reference_handle()
                 media_obj = self.db.get_object_from_handle(object_handle)
 
-                EditMediaRef(self.dbstate, self.uistate, self.track, 
-                             media_obj, media_ref, self._image_callback)
+                try:
+                    EditMediaRef(self.dbstate, self.uistate, self.track, 
+                                 media_obj, media_ref, self._image_callback)
+                except Errors.WindowActiveError, msg:
+                    pass
+
         elif event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
             media_list = self.obj.get_media_list()
             if media_list:
