@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _PlaceSidebarFilter.py 7233 2006-08-22 16:49:21Z rshura $
+# $Id: _PlaceSidebarFilter.py 7866 2007-01-04 05:09:41Z dallingham $
 
 #-------------------------------------------------------------------------
 #
@@ -102,9 +102,6 @@ class PlaceSidebarFilter(SidebarFilter):
         self.filter_note.set_text('')
         self.generic.set_active(0)
 
-    def clicked(self, obj):
-        self.clicked_func()
-
     def get_filter(self):
         gid = unicode(self.filter_id.get_text()).strip()
         title = unicode(self.filter_title.get_text()).strip()
@@ -143,8 +140,8 @@ class PlaceSidebarFilter(SidebarFilter):
 
 	    if self.generic.get_active() != 0:
 		model = self.generic.get_model()
-		iter = self.generic.get_active_iter()
-		obj = model.get_value(iter, 0)
+		node = self.generic.get_active_iter()
+		obj = model.get_value(node, 0)
 		rule = MatchesFilter([obj])
 		generic_filter.add_rule(rule)
 
