@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _GrampsBSDDB.py 7707 2006-11-26 19:47:34Z rshura $
+# $Id: _GrampsBSDDB.py 7963 2007-01-23 17:02:55Z dallingham $
 
 """
 Provides the Berkeley DB (BSDDB) database backend for GRAMPS
@@ -34,7 +34,6 @@ import os
 import shutil
 import re
 import time
-import locale
 from gettext import gettext as _
 from bsddb import dbshelve, db
 import logging
@@ -60,7 +59,7 @@ import Errors
 from BasicUtils import UpdateCallback
 
 _MINVERSION = 5
-_DBVERSION = 11
+_DBVERSION = 12
 
 def find_surname(key,data):
     return str(data[3][5])
@@ -356,8 +355,10 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                 env_flags = env_flags | db.DB_RECOVER
 
             # Environment name is now based on the filename
+            drive, tmp_name = os.path.splitdrive(self.full_name)
+            tmp_name = tmp_name.lstrip(os.sep)
             env_name = os.path.join(os.path.expanduser(const.env_dir),
-                                    self.full_name[1:])
+                                    tmp_name)
 
             # Create the env dir if it does not exist
             if not os.path.isdir(env_name):
@@ -474,19 +475,19 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                 format = format + (True,)
                 self.name_formats[format_ix] = format
         # bookmarks
-        self.bookmarks = self.metadata.get('bookmarks',default=[])
-        self.family_bookmarks = self.metadata.get('family_bookmarks',
-                                                  default=[])
-        self.event_bookmarks = self.metadata.get('event_bookmarks',
-                                                 default=[])
-        self.source_bookmarks = self.metadata.get('source_bookmarks',
-                                                  default=[])
-        self.repo_bookmarks = self.metadata.get('repo_bookmarks',
-                                                default=[])
-        self.media_bookmarks = self.metadata.get('media_bookmarks',
-                                                 default=[])
-        self.place_bookmarks = self.metadata.get('place_bookmarks',
-                                                 default=[])
+        self.bookmarks.set(self.metadata.get('bookmarks',default=[]))
+        self.family_bookmarks.set(self.metadata.get('family_bookmarks',
+                                                    default=[]))
+        self.event_bookmarks.set(self.metadata.get('event_bookmarks',
+                                                   default=[]))
+        self.source_bookmarks.set(self.metadata.get('source_bookmarks',
+                                                    default=[]))
+        self.repo_bookmarks.set(self.metadata.get('repo_bookmarks',
+                                                  default=[]))
+        self.media_bookmarks.set(self.metadata.get('media_bookmarks',
+                                                   default=[]))
+        self.place_bookmarks.set(self.metadata.get('place_bookmarks',
+                                                   default=[]))
         # Custom type values
         self.family_event_names = set(self.metadata.get('fevent_names',
                                                         default=[]))
@@ -511,6 +512,8 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
         self.url_types = set(self.metadata.get('url_types',default=[]))
         self.media_attributes = set(self.metadata.get('mattr_names',
                                                       default=[]))
+        # surname list
+        self.surname_list = self.metadata.get('surname_list',default=[])
 
     def connect_secondary(self):
         """
@@ -603,8 +606,16 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                                          table_flags)
         self.secondary_connected = True
 
+        self.smap_index = len(self.source_map)
+        self.emap_index = len(self.event_map)
+        self.pmap_index = len(self.person_map)
+        self.fmap_index = len(self.family_map)
+        self.lmap_index = len(self.place_map)
+        self.omap_index = len(self.media_map)
+        self.rmap_index = len(self.repository_map)
 
-    def rebuild_secondary(self,callback=None):
+
+    def rebuild_secondary(self,callback):
         if self.readonly:
             return
 
@@ -614,54 +625,65 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
         self.id_trans.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"idtrans")
+        callback(1)
 
         self.surnames.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"surnames")
+        callback(2)
 
         # Repair secondary indices related to family_map
         self.fid_trans.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"fidtrans")
+        callback(3)
 
         # Repair secondary indices related to place_map
         self.pid_trans.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"pidtrans")
+        callback(4)
 
         # Repair secondary indices related to media_map
         self.oid_trans.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"oidtrans")
+        callback(5)
 
         # Repair secondary indices related to source_map
         self.sid_trans.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"sidtrans")
+        callback(6)
 
         # Repair secondary indices related to event_map
         self.eid_trans.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"eidtrans")
+        callback(7)
 
         # Repair secondary indices related to repository_map
         self.rid_trans.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"ridtrans")
+        callback(8)
 
         # Repair secondary indices related to reference_map
         self.reference_map_primary_map.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"reference_map_primary_map")
+        callback(9)
 
         self.reference_map_referenced_map.close()
         junk = db.DB(self.env)
         junk.remove(self.full_name,"reference_map_referenced_map")
+        callback(10)
 
         # Set flag saying that we have removed secondary indices
         # and then call the creating routine
         self.secondary_connected = False
         self.connect_secondary()
+        callback(11)
 
     def find_backlink_handles(self, handle, include_classes=None):
         """
@@ -863,22 +885,42 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
             transaction.add(REFERENCE_KEY,str(key),None,data)
             transaction.reference_add.append((str(key),data))
 
-    def reindex_reference_map(self):
+    def reindex_reference_map(self,callback):
         """
         Reindex all primary records in the database.
         This will be a slow process for large databases.
 
-        At present this method does not clear the reference_map before it
-        reindexes. This is fine when if reindex is run to index new content
-        or when upgrading from a non-reference_map version of the database.
-        But it might be a problem if reindex is used to repair a broken index
-        because any references to primary objects that are no longer in the
-        database will remain in the reference_map index.
-        
-        So if you want to reindex for repair purposes you need to
-        clear the reference_map first.
         """
 
+        # First, remove the reference map and related tables
+        self.reference_map_referenced_map.close()
+        junk = db.DB(self.env)
+        junk.remove(self.full_name,"reference_map_referenced_map")
+        callback(1)
+
+        self.reference_map_primary_map.close()
+        junk = db.DB(self.env)
+        junk.remove(self.full_name,"reference_map_primary_map")
+        callback(2)
+
+        self.reference_map.close()
+        junk = db.DB(self.env)
+        junk.remove(self.full_name,"reference_map")
+        callback(3)
+
+        # Open reference_map and primapry map
+        self.reference_map  = self.open_table(self.full_name, "reference_map",
+                                              dbtype=db.DB_BTREE)
+        
+        open_flags = self.open_flags()
+        self.reference_map_primary_map = db.DB(self.env)
+        self.reference_map_primary_map.set_flags(db.DB_DUP)
+        self.reference_map_primary_map.open(self.full_name,
+                                            "reference_map_primary_map",
+                                            db.DB_BTREE, flags=open_flags)
+        self.reference_map.associate(self.reference_map_primary_map,
+                                     find_primary_handle,
+                                     open_flags)
 
         # Make a dictionary of the functions and classes that we need for
         # each of the primary object tables.
@@ -899,6 +941,9 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                            'class_func': Repository},
             }
 
+        transaction = self.transaction_begin(batch=True,no_magic=True)
+        callback(4)
+
         # Now we use the functions and classes defined above
         # to loop through each of the primary object tables.
         for primary_table_name in primary_tables.keys():
@@ -909,17 +954,35 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
             # Grab the real object class here so that the lookup does
             # not happen inside the cursor loop.
             class_func = primary_tables[primary_table_name]['class_func']
-
             while data:
                 found_handle,val = data
                 obj = class_func()
                 obj.unserialize(val)
 
-                self._update_reference_map(obj,transaction)
+                if self.UseTXN:
+                    the_txn = self.env.txn_begin()
+                else:
+                    the_txn = None
+                self._update_reference_map(obj,transaction,the_txn)
+                if not self.UseTXN:
+                    data_map.sync()
+                if the_txn:
+                    the_txn.commit()
                 
                 data = cursor.next()
 
             cursor.close()
+        callback(5)
+        self.transaction_commit(transaction,_("Rebuild reference map"))
+
+        self.reference_map_referenced_map = db.DB(self.env)
+        self.reference_map_referenced_map.set_flags(db.DB_DUP|db.DB_DUPSORT)
+        self.reference_map_referenced_map.open(
+            self.full_name,"reference_map_referenced_map",
+            db.DB_BTREE,flags=open_flags)
+        self.reference_map.associate(self.reference_map_referenced_map,
+                                     find_referenced_handle,open_flags)
+        callback(6)
 
         return
         
@@ -933,19 +996,22 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
 
             # name display formats
             self.metadata.put('name_formats',self.name_formats,txn=the_txn)
+
             # bookmarks
-            self.metadata.put('bookmarks',self.bookmarks,txn=the_txn)
-            self.metadata.put('family_bookmarks',self.family_bookmarks,
+            self.metadata.put('bookmarks',self.bookmarks.get(),txn=the_txn)
+            self.metadata.put('family_bookmarks',self.family_bookmarks.get(),
                               txn=the_txn)
-            self.metadata.put('event_bookmarks',self.event_bookmarks,
+            self.metadata.put('event_bookmarks',self.event_bookmarks.get(),
                               txn=the_txn)
-            self.metadata.put('source_bookmarks',self.source_bookmarks,
+            self.metadata.put('source_bookmarks',self.source_bookmarks.get(),
                               txn=the_txn)
-            self.metadata.put('place_bookmarks',self.place_bookmarks,
+            self.metadata.put('place_bookmarks',self.place_bookmarks.get(),
                               txn=the_txn)
-            self.metadata.put('repo_bookmarks',self.repo_bookmarks,txn=the_txn)
-            self.metadata.put('media_bookmarks',self.media_bookmarks,
+            self.metadata.put('repo_bookmarks',self.repo_bookmarks.get(),
                               txn=the_txn)
+            self.metadata.put('media_bookmarks',self.media_bookmarks.get(),
+                              txn=the_txn)
+
             # gender stats
             self.metadata.put('gender_stats',self.genderStats.save_stats(),
                               txn=the_txn)
@@ -976,6 +1042,9 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                               txn=the_txn)
             self.metadata.put('mattr_names',list(self.media_attributes),
                               txn=the_txn)
+            # name display formats
+            self.metadata.put('surname_list',self.surname_list,txn=the_txn)
+
             if self.UseTXN:
                 the_txn.commit()
             else:
@@ -1033,10 +1102,12 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
 
         # Attempt to clear log sequence numbers, to make database portable
         # This will only work for python2.5 and higher
-        try:
-            self.env.lsn_reset(self.full_name)
-        except AttributeError:
-            pass
+# Comment this our because it causes crashes.
+# To reproduce the crash, create a new DB, import example.gramps, open and close the db a few times.
+#        try:
+#            self.env.lsn_reset(self.full_name)
+#        except AttributeError:
+#            pass
         
         self.env.close()
 
@@ -1134,11 +1205,22 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                 self.name_group.sync()
             self.emit('person-rebuild')
 
-    def get_surname_list(self):
-        vals = [ (locale.strxfrm(unicode(val)),unicode(val))
-                 for val in set(self.surnames.keys()) ]
-        vals.sort()
-        return [item[1] for item in vals]
+    def build_surname_list(self):
+        self.surname_list = list(set(self.surnames.keys()))
+        self.sort_surname_list()
+
+    def remove_from_surname_list(self,person):
+        """
+        Check whether there are persons with the same surname left in
+        the database. If not then we need to remove the name from the list.
+        The function must be overridden in the derived class.
+        """
+        name = str(person.get_primary_name().get_surname())
+        try:
+            if self.surnames.keys().count(name) == 1:
+                self.surname_list.remove(unicode(name))
+        except ValueError:
+            pass
 
     def _get_obj_from_gramps_id(self,val,tbl,class_init,prim_tbl):
         if tbl.has_key(str(val)):
@@ -1365,6 +1447,10 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                     db.DB_BTREE,flags=open_flags)
                 self.reference_map.associate(self.reference_map_referenced_map,
                                              find_referenced_handle,open_flags)
+
+            # Only build surname list after surname index is surely back
+            self.build_surname_list()
+
         self.txn = None
 
     def undo(self,update_history=True):
@@ -1435,6 +1521,8 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
                 self.gramps_upgrade_10()
             if version < 11:
                 self.gramps_upgrade_11()
+            if version < 12:
+                self.gramps_upgrade_12()
         print "Upgrade time:", int(time.time()-t), "seconds"
 
     def gramps_upgrade_6(self):
@@ -1529,7 +1617,7 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
         to commit the objects, because the unpickled objects will lack
         some attributes that are necessary for serialization on commit.
         """
-        print "Upgrading to DB version 11 -- this may take a while"
+        print "Upgrading to DB version 12 -- this may take a while"
         # The very very first thing is to check for duplicates in the
         # primary tables and remove them. 
         self.set_total(7)
@@ -1613,6 +1701,11 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
             (junk_handle, source.gramps_id, source.title, source.author,
              source.pubinfo, source.note, source.media_list,
              source.abbrev, source.change, source.datamap) = info
+
+            # Cover attributes contained in MediaRefs
+            for media_ref in source.media_list:
+                convert_mediaref_9(media_ref)
+
             self.commit_source(source,trans)
             self.update()
 
@@ -1873,7 +1966,7 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
             place = Place()
             place.handle = handle
             (junk_handle, place.gramps_id, place.title, place.long, place.lat,
-             place.main_loc, alt_loc, place.urls, place.media_list,
+             place.main_loc, place.alt_loc, place.urls, place.media_list,
              place.source_list, place.note, place.change) = info
 
             # Cover attributes contained in MediaRefs
@@ -1910,7 +2003,7 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
             self.commit_media_object(media_object,trans)
             self.update()
 
-        self.transaction_commit(trans,"Upgrade to DB version 9")
+        self.transaction_commit(trans,"Upgrade to DB version 12")
         # Close secodnary index
         self.reference_map_primary_map.close()
 
@@ -1919,13 +2012,13 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
             the_txn = self.env.txn_begin()
         else:
             the_txn = None
-        self.metadata.put('version',11,txn=the_txn)
+        self.metadata.put('version',12,txn=the_txn)
         if self.UseTXN:
             the_txn.commit()
         else:
             self.metadata.sync()
 
-        print "Done upgrading to DB version 11"
+        print "Done upgrading to DB version 12"
 
     def gramps_upgrade_10(self):
         print "Upgrading to DB version 10..."
@@ -2179,6 +2272,35 @@ class GrampsBSDDB(GrampsDbBase,UpdateCallback):
 
         print "Done upgrading to DB version 11"
 
+    def gramps_upgrade_12(self):
+        print "Upgrading to DB version 12..."
+        # Hook up surnames 
+        table_flags = self.open_flags()
+        self.surnames = db.DB(self.env)
+        self.surnames.set_flags(db.DB_DUP|db.DB_DUPSORT)
+        self.surnames.open(self.full_name, "surnames", db.DB_BTREE,
+                           flags=table_flags)
+        self.person_map.associate(self.surnames, find_surname, table_flags)
+        
+        self.build_surname_list()
+
+        # Close so that we can open it again later
+        self.surnames.close()
+
+        if self.UseTXN:
+            # Separate transaction to save metadata
+            the_txn = self.env.txn_begin()
+        else:
+            the_txn = None
+        self.metadata.put('version', 12, txn=the_txn)
+        if self.UseTXN:
+            the_txn.commit()
+        else:
+            self.metadata.sync()
+
+        print "Done upgrading to DB version 12"
+
+
 class BdbTransaction(Transaction):
     def __init__(self,msg,db,batch=False,no_magic=False):
         Transaction.__init__(self,msg,db,batch,no_magic)
