@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _GrampsDbBase.py 7481 2006-10-29 04:36:08Z rshura $
+# $Id: _GrampsDbBase.py 7983 2007-01-25 17:04:37Z rshura $
 
 """
 Base class for the GRAMPS databases. All database interfaces should inherit
@@ -26,7 +26,7 @@ from this class.
 """
 
 __author__ = "Donald N. Allingham"
-__revision__ = "$Revision: 7481 $"
+__revision__ = "$Revision: 7983 $"
 
 #-------------------------------------------------------------------------
 #
@@ -154,6 +154,31 @@ class GrampsCursor:
         """
         pass
 
+class GrampsDbBookmarks:
+    def __init__(self, default = []):
+        self.bookmarks = default
+
+    def set(self, new_list):
+        self.bookmarks = new_list
+
+    def get(self):
+        return self.bookmarks
+
+    def append(self, item):
+        self.bookmarks.append(item)
+
+    def append_list(self, blist):
+        self.bookmarks += blist
+
+    def remove(self, item):
+        self.bookmarks.remove(item)
+
+    def pop(self, item):
+        self.bookmarks.pop(item)
+
+    def insert(self, item):
+        self.bookmarks.insert(item)
+
 class GrampsDbBase(GrampsDBCallback):
     """
     GRAMPS database object. This object is a base class for all
@@ -273,17 +298,18 @@ class GrampsDbBase(GrampsDBCallback):
         self.default = None
         self.owner = Researcher()
         self.name_formats = []
-        self.bookmarks = []
-        self.family_bookmarks = []
-        self.event_bookmarks = []
-        self.place_bookmarks = []
-        self.source_bookmarks = []
-        self.repo_bookmarks = []
-        self.media_bookmarks = []
+        self.bookmarks = GrampsDbBookmarks()
+        self.family_bookmarks = GrampsDbBookmarks()
+        self.event_bookmarks = GrampsDbBookmarks()
+        self.place_bookmarks = GrampsDbBookmarks()
+        self.source_bookmarks = GrampsDbBookmarks()
+        self.repo_bookmarks = GrampsDbBookmarks()
+        self.media_bookmarks = GrampsDbBookmarks()
         self.path = ""
         self.name_group = {}
+        self.surname_list = []
 
-    def rebuild_secondary(self, callback=None):
+    def rebuild_secondary(self, callback):
         pass
 
     def version_supported(self):
@@ -447,11 +473,15 @@ class GrampsDbBase(GrampsDBCallback):
         if old_data:
             old_person = Person(old_data)
             if (old_data[2] != person.gender or
-                old_data[3][2]!= person.primary_name.first_name):
+                old_data[3][4]!= person.primary_name.first_name):
                 self.genderStats.uncount_person(old_person)
                 self.genderStats.count_person(person)
+            if (old_data[3][5]!=person.primary_name.surname):
+                self.remove_from_surname_list(old_person)
+                self.add_to_surname_list(person,transaction.batch)
         else:
             self.genderStats.count_person(person)
+            self.add_to_surname_list(person,transaction.batch)
 
         self.individual_attributes.update(
             [str(attr.type) for attr in person.attribute_list
@@ -1337,12 +1367,13 @@ class GrampsDbBase(GrampsDBCallback):
         """
         Commits the transaction to the assocated UNDO database.
         """
-        
         if self.__LOG_ALL:
             log.debug("%s: Transaction commit '%s'\n"
                       % (self.__class__.__name__, str(msg)))
+
         if not len(transaction) or self.readonly:
             return
+
         transaction.set_description(msg)
         transaction.timestamp = time.time()
         self.undoindex += 1                    
@@ -1562,7 +1593,34 @@ class GrampsDbBase(GrampsDBCallback):
 
     def get_surname_list(self):
         """
-        Returns the list of surnames contained within the database.
+        Returns the list of locale-sorted surnames contained in the database.
+        """
+        return self.surname_list
+
+    def build_surname_list(self):
+        """
+        Builds the list of locale-sorted surnames contained in the database.
+        The function must be overridden in the derived class.
+        """
+        assert False, "Needs to be overridden in the derived class"
+
+    def sort_surname_list(self):
+        vals = [(locale.strxfrm(item),item) for item in self.surname_list]
+        vals.sort()
+        self.surname_list = [item[1] for item in vals]
+
+    def add_to_surname_list(self,person,batch_transaction):
+        if batch_transaction:
+            return
+        name = unicode(person.get_primary_name().get_surname())
+        if name not in self.surname_list:
+            self.surname_list.append(name)
+            self.sort_surname_list()
+
+    def remove_from_surname_list(self,person):
+        """
+        Check whether there are persons with the same surname left in
+        the database. If not then we need to remove the name from the list.
         The function must be overridden in the derived class.
         """
         assert False, "Needs to be overridden in the derived class"
@@ -1609,7 +1667,7 @@ class GrampsDbBase(GrampsDBCallback):
 
     def set_default_person_handle(self, handle):
         """sets the default Person to the passed instance"""
-        if not self.readonly:
+        if (self.metadata != None) and (not self.readonly):
             self.metadata['default'] = str(handle)
 
     def get_default_person(self):
@@ -1617,13 +1675,13 @@ class GrampsDbBase(GrampsDBCallback):
         person = self.get_person_from_handle(self.get_default_handle())
         if person:
             return person
-        elif (self.metadata) and (not self.readonly):
+        elif (self.metadata != None) and (not self.readonly):
             self.metadata['default'] = None
         return None
 
     def get_default_handle(self):
         """returns the default Person of the database"""
-        if self.metadata:
+        if self.metadata != None:
             return self.metadata.get('default')
         return None
 
@@ -1711,6 +1769,7 @@ class GrampsDbBase(GrampsDBCallback):
         self._delete_primary_from_reference_map(handle, transaction)
         person = self.get_person_from_handle(handle)
         self.genderStats.uncount_person (person)
+        self.remove_from_surname_list(person)
         if transaction.batch:
             self._del_person(handle)
         else:
@@ -1875,7 +1934,7 @@ class GrampsDbBase(GrampsDBCallback):
         return locale.strcoll(media1, media2)
 
     def _set_column_order(self, col_list, name):
-        if self.metadata and not self.readonly: 
+        if (self.metadata != None) and (not self.readonly):
             self.metadata[name] = col_list
 
     def set_person_column_order(self, col_list):
@@ -2044,6 +2103,13 @@ class GrampsDbBase(GrampsDBCallback):
         need to be changed."""
         pass
 
+    def reindex_reference_map(self,callback):
+        """
+        Reindex all primary records in the database.
+
+        """
+        pass
+
     def find_backlink_handles(self, handle, include_classes=None):
         """
         Find all objects that hold a reference to the object handle.
@@ -2086,9 +2152,15 @@ class GrampsDbBase(GrampsDBCallback):
             }
 
 
+        # Find which tables to iterate over
+        if (include_classes == None):
+            the_tables = primary_tables.keys()
+        else:
+            the_tables = include_classes
+        
         # Now we use the functions and classes defined above to loop through
         # each of the existing primary object tables
-        for primary_table_name in primary_tables.keys():
+        for primary_table_name in the_tables:
             cursor = primary_tables[primary_table_name]['cursor_func']()
             data = cursor.first()
 
@@ -2103,12 +2175,9 @@ class GrampsDbBase(GrampsDBCallback):
 
                 # Now we need to loop over all object types
                 # that have been requests in the include_classes list
-                for classname in primary_tables.keys():
-                    if (include_classes == None) \
-                           or (classname in include_classes):
-                
-                        if obj.has_handle_reference(classname,handle):
-                            yield (primary_table_name, found_handle)
+                for classname in primary_tables.keys():               
+                    if obj.has_handle_reference(classname,handle):
+                        yield (primary_table_name, found_handle)
                         
                 data = cursor.next()
                     
