@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: _WriteGedcom.py 7716 2006-11-27 03:20:37Z rshura $
+# $Id: _WriteGedcom.py 7993 2007-01-27 05:30:15Z dallingham $
 
 "Export to GEDCOM"
 
@@ -60,7 +60,7 @@ import Errors
 import ansel_utf8
 import Utils
 import NameDisplay
-from QuestionDialog import ErrorDialog, WarningDialog, MessageHideDialog
+from QuestionDialog import *
 from BasicUtils import UpdateCallback
 
 #------------------------------------------------------------------------
@@ -268,7 +268,7 @@ class GedcomWriterOptionBox:
     def get_option_box(self):
         self.restrict = True
         self.private = True
-        self.cnvtxt = ansel_utf8.utf8_to_ansel
+        self.cnvtxt = keep_utf8
         self.adopt = GedcomInfo.ADOPT_EVENT
 
         glade_file = "%s/gedcomexport.glade" % os.path.dirname(__file__)
@@ -286,6 +286,8 @@ class GedcomWriterOptionBox:
                 "on_restrict_toggled": self.on_restrict_toggled,
                 })
 
+        self.topDialog.get_widget("encoding").set_history(1)
+
         filter_obj = self.topDialog.get_widget("filter")
         self.copy = 0
 
@@ -1059,9 +1061,9 @@ class GedcomWriter(UpdateCallback):
                 else:
                     self.writeln("1 EVEN")
                     if value:
-                        self.writeln("2 TYPE %s %s" %(self.cnvtxt(name),value))
+                        self.writeln("2 TYPE %s %s" %(self.cnvtxt(key),value))
                     else:
-                        self.writeln("2 TYPE %s" % self.cnvtxt(name))
+                        self.writeln("2 TYPE %s" % self.cnvtxt(key))
 
                 if attr.get_note():
                     self.write_long_text("NOTE",2,self.cnvtxt(attr.get_note()))
@@ -1385,13 +1387,12 @@ class GedcomWriter(UpdateCallback):
             # Reference to the source
             self.writeln("%d SOUR @%s@" % (level,src.get_gramps_id()))
             if ref.get_page() != "":
-                self.write_long_text("PAGE",level+1,
-                                     self.cnvtxt(ref.get_page()))
-
+                page_text = self.cnvtxt(ref.get_page())
+                self.writeln('%d PAGE %s' % (level+1,page_text))
             conf = ref.get_confidence_level()
             # Cap the maximum level
             conf = min(conf,RelLib.SourceRef.CONF_VERY_HIGH)
-            if conf != RelLib.SourceRef.CONF_NORMAL:
+            if conf != RelLib.SourceRef.CONF_NORMAL and conf != -1:
                 self.write_long_text("QUAY",level+1, str(quay_map[conf]))
 
             ref_text = ref.get_text()
@@ -1490,7 +1491,9 @@ def exportData(database,filename,person,option_box,callback=None):
     try:
         gw = GedcomWriter(database,person,0,filename,option_box,callback)
         ret = gw.export_data(filename)
-    except Errors.DatabaseError,msg:
+    except AttributeError, msg:
+        RunDatabaseRepair(msg)
+    except Errors.DatabaseError, msg:
         ErrorDialog(_("Export failed"),str(msg))
     return ret
 
