@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _WriteXML.py 7689 2006-11-25 01:02:26Z dallingham $
+# $Id: _WriteXML.py 7837 2006-12-22 19:24:46Z dallingham $
 
 """
 Contains the interface to allow a database to get written using
@@ -34,6 +34,7 @@ import time
 import shutil
 import os
 import codecs
+from xml.sax.saxutils import escape
 from gettext import gettext as _
 
 #------------------------------------------------------------------------
@@ -75,6 +76,10 @@ _xml_version = "1.1.3"
 # table for skipping control chars from XML
 strip_dict = dict.fromkeys(range(9)+range(12,20))
 
+
+def escxml(d):
+    return escape(d, { '"' : '&quot;' } )
+
 #-------------------------------------------------------------------------
 #
 #
@@ -320,13 +325,13 @@ class XmlWriter(UpdateCallback):
         self.g.write("</database>\n")
 
     def write_bookmarks(self):
-        bm_person_len = len(self.db.bookmarks)
-        bm_family_len = len(self.db.family_bookmarks)
-        bm_event_len = len(self.db.event_bookmarks)
-        bm_source_len = len(self.db.source_bookmarks)
-        bm_place_len = len(self.db.place_bookmarks)
-        bm_repo_len = len(self.db.repo_bookmarks)
-        bm_obj_len = len(self.db.media_bookmarks)
+        bm_person_len = len(self.db.bookmarks.get())
+        bm_family_len = len(self.db.family_bookmarks.get())
+        bm_event_len = len(self.db.event_bookmarks.get())
+        bm_source_len = len(self.db.source_bookmarks.get())
+        bm_place_len = len(self.db.place_bookmarks.get())
+        bm_repo_len = len(self.db.repo_bookmarks.get())
+        bm_obj_len = len(self.db.media_bookmarks.get())
 
         bm_len = bm_person_len + bm_family_len + bm_event_len \
                  + bm_source_len + bm_place_len + bm_repo_len + bm_obj_len
@@ -334,25 +339,25 @@ class XmlWriter(UpdateCallback):
         if bm_len > 0:
             self.g.write("  <bookmarks>\n")
 
-            for handle in self.db.get_bookmarks():
+            for handle in self.db.get_bookmarks().get():
                 self.g.write('    <bookmark target="person" hlink="_%s"/>\n'
                              % handle )
-            for handle in self.db.get_family_bookmarks():
+            for handle in self.db.get_family_bookmarks().get():
                 self.g.write('    <bookmark target="family" hlink="_%s"/>\n'
                              % handle )
-            for handle in self.db.get_event_bookmarks():
+            for handle in self.db.get_event_bookmarks().get():
                 self.g.write('    <bookmark target="event" hlink="_%s"/>\n'
                              % handle )
-            for handle in self.db.get_source_bookmarks():
+            for handle in self.db.get_source_bookmarks().get():
                 self.g.write('    <bookmark target="source" hlink="_%s"/>\n'
                              % handle )
-            for handle in self.db.get_place_bookmarks():
+            for handle in self.db.get_place_bookmarks().get():
                 self.g.write('    <bookmark target="place" hlink="_%s"/>\n'
                              % handle )
-            for handle in self.db.get_media_bookmarks():
+            for handle in self.db.get_media_bookmarks().get():
                 self.g.write('    <bookmark target="media" hlink="_%s"/>\n'
                              % handle )
-            for handle in self.db.get_repo_bookmarks():
+            for handle in self.db.get_repo_bookmarks().get():
                 self.g.write('    <bookmark target="repository" hlink="_%s"/>\n'
                              % handle )
             self.g.write("  </bookmarks>\n")
@@ -372,10 +377,7 @@ class XmlWriter(UpdateCallback):
         except:
             l = unicode(str(line),errors='replace')
         l = l.strip().translate(strip_dict)
-        l = l.replace('&','&amp;')
-        l = l.replace('>','&gt;')
-        l = l.replace('<','&lt;')
-        return l.replace('"','&quot;')
+        return escxml(l)
 
     def write_note(self,val,noteobj,indent=0):
         if not noteobj:
@@ -524,7 +526,7 @@ class XmlWriter(UpdateCallback):
             return
         sp = "  "*index
         priv_text = conf_priv(personref)
-        rel_text = ' rel="%s"'
+        rel_text = ' rel="%s"' % escxml(personref.get_relation())
 
         sreflist = personref.get_source_references()
         if (len(sreflist) == 0) and personref.get_note() =="":
@@ -546,11 +548,11 @@ class XmlWriter(UpdateCallback):
         if childref.frel.is_default():
             frel_text = ''
         else:
-            frel_text = ' frel="%s"' % childref.frel.xml_str()
+            frel_text = ' frel="%s"' % escxml(childref.frel.xml_str())
         if childref.mrel.is_default():
             mrel_text = ''
         else:
-            mrel_text = ' mrel="%s"' % childref.mrel.xml_str()
+            mrel_text = ' mrel="%s"' % escxml(childref.mrel.xml_str())
         sreflist = childref.get_source_references()
         if (len(sreflist) == 0) and childref.get_note() =="":
             self.write_ref('childref',childref.ref,index,close=True,
@@ -568,7 +570,7 @@ class XmlWriter(UpdateCallback):
             return
         sp = "  "*index
         priv_text = conf_priv(eventref)
-        role = eventref.role.xml_str()
+        role = escxml(eventref.role.xml_str())
         if role:
             role_text = ' role="%s"' % role
         else:
@@ -670,7 +672,7 @@ class XmlWriter(UpdateCallback):
         sp = "  "*index
         marker = obj.get_marker().xml_str()
         if marker:
-            marker_text = ' marker="%s"' % marker
+            marker_text = ' marker="%s"' % escxml(marker)
         else:
             marker_text = ''
         priv_text = conf_priv(obj)
@@ -685,7 +687,7 @@ class XmlWriter(UpdateCallback):
         sp = "  "*index
         self.write_primary_tag('family',family,index)
         if family:
-            rel = family.get_relationship().xml_str()
+            rel = escxml(family.get_relationship().xml_str())
             if rel != "":
                 self.g.write('  %s<rel type="%s"/>\n' % (sp,rel) )
 
@@ -695,9 +697,9 @@ class XmlWriter(UpdateCallback):
         g = name.get_group_as()
         self.g.write('%s<last' % ('  '*indent))
         if p:
-            self.g.write(' prefix="%s"' % p)
+            self.g.write(' prefix="%s"' % escxml(p))
         if g:
-            self.g.write(' group="%s"' % g)
+            self.g.write(' group="%s"' % escxml(g))
         self.g.write('>%s</last>\n' % self.fix(n))
 
     def write_line(self,tagname,value,indent=1):
@@ -785,7 +787,7 @@ class XmlWriter(UpdateCallback):
         if alternative:
             self.g.write(' alt="1"')
         if name_type:
-            self.g.write(' type="%s"' % name_type)
+            self.g.write(' type="%s"' % escxml(name_type))
         if name.get_privacy() != 0:
             self.g.write(' priv="%d"' % name.get_privacy())
         if name.get_sort_as() != 0:
@@ -875,7 +877,7 @@ class XmlWriter(UpdateCallback):
         sp = '  ' * indent
         for attr in list:
             self.g.write('%s<attribute%s type="%s" value="%s"' % \
-                         (sp,conf_priv(attr),attr.get_type().xml_str(),
+                         (sp,conf_priv(attr),escxml(attr.get_type().xml_str()),
                          self.fix(attr.get_value())))
             slist = attr.get_source_references()
             note = attr.get_note()
@@ -925,11 +927,11 @@ class XmlWriter(UpdateCallback):
             if reporef.call_number == "":
                 callno_text = ''
             else:
-                callno_text = ' callno="%s"' % reporef.call_number
+                callno_text = ' callno="%s"' % escxml(reporef.call_number)
                 
             mtype = reporef.media_type.xml_str()
             if mtype:
-                type_text = ' medium="%s"' % mtype
+                type_text = ' medium="%s"' % escxml(mtype)
             else:
                 type_text = ''
 
@@ -948,7 +950,7 @@ class XmlWriter(UpdateCallback):
         for url in list:
             url_type = url.get_type().xml_str()
             if url_type:
-                type_text = ' type="%s"' % url_type
+                type_text = ' type="%s"' % escxml(url_type)
             else:
                 type_text = ''
             priv_text = conf_priv(url)
