@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: PageView.py 7581 2006-11-08 05:15:26Z dallingham $
+# $Id: PageView.py 7999 2007-01-28 04:38:56Z dallingham $
 
 #----------------------------------------------------------------
 #
@@ -71,6 +71,7 @@ class PageView:
         self.additional_action_groups = []
         self.additional_uis = []
         self.widget = None
+        self.model = None
         self.ui = '<ui></ui>'
         self.dbstate.connect('no-database',self.disable_action_group)
         self.dbstate.connect('database-changed',self.enable_action_group)
@@ -78,7 +79,6 @@ class PageView:
         self.active = False
         self.handle_col = 0
         self.selection = None
-        self.func_list = {}
 
     def call_function(self, key):
         self.func_list.get(key)()
@@ -241,7 +241,7 @@ class BookMarkView(PageView):
     def enable_action_group(self, obj):
         PageView.enable_action_group(self, obj)
 
-    def disable_action_group(self, obj):
+    def disable_action_group(self):
         PageView.disable_action_group(self)
 
     def define_actions(self):
@@ -279,7 +279,7 @@ class PersonNavView(BookMarkView):
         self.fwd_action = gtk.ActionGroup(self.title + '/Forward')
         self.fwd_action.add_actions([
             ('Forward',gtk.STOCK_GO_FORWARD,_("_Forward"),
-             None, _("Go to the next person in the history"),
+             "<ALT>Right", _("Go to the next person in the history"),
              self.fwd_clicked)
             ])
 
@@ -287,19 +287,25 @@ class PersonNavView(BookMarkView):
         self.back_action = gtk.ActionGroup(self.title + '/Backward')
         self.back_action.add_actions([
             ('Back',gtk.STOCK_GO_BACK,_("_Back"),
-             None, _("Go to the previous person in the history"),
+             "<ALT>Left", _("Go to the previous person in the history"),
              self.back_clicked)
             ])
 
-        self.add_action('HomePerson', gtk.STOCK_HOME, _("_Home"),
+        self.add_action('HomePerson', gtk.STOCK_HOME, _("_Home"), 
+                        accel="<Alt>Home",
                         tip=_("Go to the default person"), callback=self.home)
-        self.add_action('SetActive', gtk.STOCK_HOME, _("Set _Home Person"),
-                        callback=self.set_default_person)
-        self.add_action('FilterEdit', None, _('Person Filter Editor'),
-                        callback=self.filter_editor,)
+        self.add_action('FilterEdit',  None, _('Person Filter Editor'), 
+                        callback=self.filter_editor)
+
+        self.other_action = gtk.ActionGroup(self.title + '/PersonOther')
+        self.other_action.add_actions([
+                ('SetActive', gtk.STOCK_HOME, _("Set _Home Person"), None, 
+                 None, self.set_default_person),
+                ])
 
         self.add_action_group(self.back_action)
         self.add_action_group(self.fwd_action)
+        self.add_action_group(self.other_action)
 
     def disable_action_group(self):
         """
@@ -336,7 +342,7 @@ class PersonNavView(BookMarkView):
         if defperson:
             self.dbstate.change_active_person(defperson)
 
-    def jumpto(self,obj):
+    def jump(self):
         dialog = gtk.Dialog(_('Jump to by GRAMPS ID'),None,
                             gtk.DIALOG_NO_SEPARATOR)
         dialog.set_border_width(12)
@@ -431,6 +437,7 @@ class PersonNavView(BookMarkView):
         hobj = self.uistate.phistory
         self.fwd_action.set_sensitive(not hobj.at_end())
         self.back_action.set_sensitive(not hobj.at_front())
+        self.other_action.set_sensitive(not self.dbstate.db.readonly)
 
 #----------------------------------------------------------------
 #
@@ -473,6 +480,16 @@ class ListView(BookMarkView):
         self.filter_toggle(None, None, None, None)
         return hpaned
 
+    def filter_toggle(self, client, cnxn_id, entry, data):
+        if Config.get(Config.FILTER):
+            self.search_bar.hide()
+            self.filter_pane.show()
+            active = True
+        else:
+            self.search_bar.show()
+            self.filter_pane.hide()
+            active = False
+
     def post(self):
         if self.filter_class:
             if Config.get(Config.FILTER):
@@ -499,6 +516,37 @@ class ListView(BookMarkView):
                 _("A bookmark could not be set because "
                   "nothing was selected."))
 
+    def jump(self):
+        dialog = gtk.Dialog(_('Jump to by GRAMPS ID'),None,
+                            gtk.DIALOG_NO_SEPARATOR)
+        dialog.set_border_width(12)
+        label = gtk.Label('<span weight="bold" size="larger">%s</span>' % _('Jump to by GRAMPS ID'))
+        label.set_use_markup(True)
+        dialog.vbox.add(label)
+        dialog.vbox.set_spacing(10)
+        dialog.vbox.set_border_width(12)
+        hbox = gtk.HBox()
+        hbox.pack_start(gtk.Label("%s: " % _('ID')),False)
+        text = gtk.Entry()
+        text.set_activates_default(True)
+        hbox.pack_start(text,False)
+        dialog.vbox.pack_start(hbox,False)
+        dialog.add_buttons(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                           gtk.STOCK_JUMP_TO, gtk.RESPONSE_OK)
+        dialog.set_default_response(gtk.RESPONSE_OK)
+        dialog.vbox.show_all()
+        
+        if dialog.run() == gtk.RESPONSE_OK:
+            gid = text.get_text()
+            handle = self.get_handle_from_gramps_id(gid)
+            if handle:
+                self.goto_handle(handle)
+            else:
+                self.uistate.push_message(
+                    self.dbstate,
+                    _("Error: %s is not a valid GRAMPS ID") % gid)
+        dialog.destroy()
+
     def drag_info(self):
         return None
 
@@ -681,7 +729,7 @@ class ListView(BookMarkView):
         else:
             self.dirty = True
         
-    def filter_toggle(self,obj):
+    def filter_toggle_action(self,obj):
         if obj.get_active():
             self.search_bar.hide()
             self.filter_pane.show()
@@ -693,18 +741,26 @@ class ListView(BookMarkView):
         Config.set(Config.FILTER, active)
         self.build_tree()
 
+    def filter_editor(self,obj):
+        from FilterEditor import FilterEditor
+
+        try:
+            FilterEditor(self.FILTER_TYPE ,const.custom_filters,
+                         self.dbstate, self.uistate)
+        except Errors.WindowActiveError:
+            pass            
+
     def change_db(self,db):
         for sig in self.signal_map:
             db.connect(sig, self.signal_map[sig])
 
-        if Config.get(Config.FILTER):
-            search = EMPTY_SEARCH
-        else:
-            search = self.search_bar.get_value()
-            
-        self.model = self.make_model(self.dbstate.db, 0, search=search)
-        
-        self.list.set_model(self.model)
+#        if Config.get(Config.FILTER):
+#            search = EMPTY_SEARCH
+#        else:
+#            search = self.search_bar.get_value()
+#        self.model = self.make_model(self.dbstate.db, 0, search=search)
+#        self.list.set_model(self.model)
+
         self.build_columns()
         self.bookmarks.update_bookmarks(self.get_bookmarks())
         if self.active:
@@ -715,13 +771,14 @@ class ListView(BookMarkView):
 
     def row_add(self,handle_list):
         if self.active:
-            for handle in handle_list:
-                self.model.add_row_by_handle(handle)
+	    for handle in handle_list:
+		self.model.add_row_by_handle(handle)
         else:
             self.dirty = True
 
     def row_update(self,handle_list):
-        self.model.prev_handle = None
+        if self.model:
+            self.model.prev_handle = None
         if self.active:
             for handle in handle_list:
                 self.model.update_row_by_handle(handle)
@@ -745,15 +802,28 @@ class ListView(BookMarkView):
         
         BookMarkView.define_actions(self)
 
-        self.add_action('Add', gtk.STOCK_ADD, _("_Add"), tip=self.ADD_MSG,
-                        callback=self.add)
-        self.add_action('Edit', gtk.STOCK_EDIT,_("_Edit"), tip=self.EDIT_MSG,
+        self.edit_action = gtk.ActionGroup(self.title + '/ChangeOrder')
+        self.edit_action.add_actions([
+                ('Add', gtk.STOCK_ADD, _("_Add"), "<control>Insert", 
+		 self.ADD_MSG, self.add),
+                ('Remove', gtk.STOCK_REMOVE, _("_Remove"), "<control>Delete", 
+		 self.DEL_MSG, self.remove),
+                ('ColumnEdit', gtk.STOCK_PROPERTIES, _('_Column Editor'), 
+		 None, None, self.column_editor),
+                ])
+
+        self.add_action_group(self.edit_action)
+
+        self.add_action('Edit', gtk.STOCK_EDIT, _("_Edit"), 
+                        accel="<control>Return",
+                        tip=self.EDIT_MSG,
                         callback=self.edit)
-        self.add_action('Remove',gtk.STOCK_REMOVE,_("_Remove"),
-                        tip=self.DEL_MSG, callback=self.remove)
         
         self.add_toggle_action('Filter', None, _('_Filter'),
-                               callback=self.filter_toggle)
+                               callback=self.filter_toggle_action)
+
+    def column_editor(self,obj):
+        pass
 
     def button_press(self,obj,event):
         if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
@@ -776,4 +846,9 @@ class ListView(BookMarkView):
     def double_click(self,obj,event):
         return False
 
+    def change_page(self):
+        self.edit_action.set_sensitive(not self.dbstate.db.readonly)
+
+    def key_delete(self):
+        self.remove(None)
 
