@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-# $Id: _PluginMgr.py 6992 2006-07-04 23:38:51Z dallingham $
+# $Id: _PluginMgr.py 7976 2007-01-25 03:22:46Z pez4brian $
 
 """
 The core of the GRAMPS plugin system. This module provides tasks to load
@@ -373,7 +373,8 @@ def register_text_doc(name,classref, table, paper, style, ext,
 # Book document generator registration
 #
 #-------------------------------------------------------------------------
-def register_book_doc(name,classref, table, paper, style, ext, clname=''):
+def register_book_doc(name,classref, table, paper, style, ext, 
+                      print_report_label=None, clname=''):
     """Register a text document generator"""
     del_index = -1
     for i in range(0,len(bookdoc_list)):
@@ -385,7 +386,8 @@ def register_book_doc(name,classref, table, paper, style, ext, clname=''):
 
     if not clname:
         clname = ext[1:]
-    bookdoc_list.append((name,classref,table,paper,style,ext,None,clname))
+    bookdoc_list.append((name,classref,table,paper,style,ext,
+                         print_report_label,clname))
 
 #-------------------------------------------------------------------------
 #
