@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: RecentFiles.py,v 1.5 2005/02/19 00:49:49 rshura Exp $
+# $Id: RecentFiles.py 6279 2006-04-06 16:50:05Z dallingham $
 
 # Written by Alex Roitman
 
@@ -28,19 +28,26 @@
 #
 #-------------------------------------------------------------------------
 import os
-import fcntl
 import time
 from  xml.parsers.expat import ParserCreate
 
+try:
+    import fcntl
+    use_lock = True
+except:
+    use_lock = False
+
+import const
+
 #-------------------------------------------------------------------------
 #
 # Constants
 #
 #-------------------------------------------------------------------------
-GNOME_FILENAME  = "~/.recently-used"
+GNOME_FILENAME  = os.path.join(const.user_home,".recently-used")
 MAX_GNOME_ITEMS = 500
 
-GRAMPS_FILENAME = "~/.gramps/recent-files.xml"
+GRAMPS_FILENAME = os.path.join(const.home_dir,"recent-files.xml")
 MAX_GRAMPS_ITEMS = 10
 
 #-------------------------------------------------------------------------
@@ -174,7 +181,8 @@ class GnomeRecentFiles:
         Saves the current GNOME RecentFiles collection to the associated file.
         """
         xml_file = file(os.path.expanduser(GNOME_FILENAME),'w')
-        fcntl.lockf(xml_file,fcntl.LOCK_EX)
+        if use_lock:
+            fcntl.lockf(xml_file,fcntl.LOCK_EX)
         xml_file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n")
         xml_file.write('<RecentFiles>\n')
         index = 0
@@ -194,7 +202,8 @@ class GnomeRecentFiles:
             xml_file.write('    </Groups>\n')
             xml_file.write('  </RecentItem>\n')
         xml_file.write('</RecentFiles>\n')
-        fcntl.lockf(xml_file,fcntl.LOCK_UN)
+        if use_lock:
+            fcntl.lockf(xml_file,fcntl.LOCK_UN)
         xml_file.close()
 
 #-------------------------------------------------------------------------
@@ -251,7 +260,8 @@ class GrampsRecentFiles:
         Saves the current GRAMPS RecentFiles collection to the associated file.
         """
         xml_file = file(os.path.expanduser(GRAMPS_FILENAME),'w')
-        fcntl.lockf(xml_file,fcntl.LOCK_EX)
+        if use_lock:
+            fcntl.lockf(xml_file,fcntl.LOCK_EX)
         xml_file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n")
         xml_file.write('<RecentFiles>\n')
         index = 0
@@ -265,7 +275,8 @@ class GrampsRecentFiles:
             xml_file.write('    <Timestamp>%d</Timestamp>\n' % item.get_time())
             xml_file.write('  </RecentItem>\n')
         xml_file.write('</RecentFiles>\n')
-        fcntl.lockf(xml_file,fcntl.LOCK_UN)
+        if use_lock:
+            fcntl.lockf(xml_file,fcntl.LOCK_UN)
         xml_file.close()
 
 #-------------------------------------------------------------------------
@@ -283,7 +294,8 @@ class GnomeRecentParser:
 
         try:
             xml_file = open(os.path.expanduser(GNOME_FILENAME))
-            fcntl.lockf(xml_file,fcntl.LOCK_SH)
+            if use_lock:
+                fcntl.lockf(xml_file,fcntl.LOCK_SH)
 
             p = ParserCreate()
             p.StartElementHandler = self.startElement
@@ -291,7 +303,8 @@ class GnomeRecentParser:
             p.CharacterDataHandler = self.characters
             p.ParseFile(xml_file)
 
-            fcntl.lockf(xml_file,fcntl.LOCK_UN)
+            if use_lock:
+                fcntl.lockf(xml_file,fcntl.LOCK_UN)
             xml_file.close()
         except:
             pass
@@ -343,7 +356,8 @@ class GrampsRecentParser:
 
         try:
             xml_file = open(os.path.expanduser(GRAMPS_FILENAME))
-            fcntl.lockf(xml_file,fcntl.LOCK_SH)
+            if use_lock:
+                fcntl.lockf(xml_file,fcntl.LOCK_SH)
 
             p = ParserCreate()
             p.StartElementHandler = self.startElement
@@ -351,7 +365,8 @@ class GrampsRecentParser:
             p.CharacterDataHandler = self.characters
             p.ParseFile(xml_file)
 
-            fcntl.lockf(xml_file,fcntl.LOCK_UN)
+            if use_lock:
+                fcntl.lockf(xml_file,fcntl.LOCK_UN)
             xml_file.close()
         except:
             pass
@@ -369,7 +384,8 @@ class GrampsRecentParser:
         text = ''.join(self.tlist)
 
         if tag == "RecentItem":
-            self.recent_files.append(self.item)
+            if os.path.isfile(self.item.get_path()):
+                self.recent_files.append(self.item)
         elif tag == "Path":
             self.item.set_path(text)
         elif tag == "Mime-Type":
@@ -394,7 +410,7 @@ def recent_files(filename,filetype):
     # Add the file to the recent items
     gnome_rf = GnomeRecentFiles()
     gnome_item = GnomeRecentItem(
-        u='file://%s' % filename,
+        u=filename,
         m=filetype,
         t=the_time,
         p=False,
