@@ -18,18 +18,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _Person.py 7668 2006-11-22 15:27:31Z loshawlos $
+# $Id: _Person.py 7979 2007-01-25 05:26:22Z rshura $
 
 """
 Person object for GRAMPS
 """
 
-#-------------------------------------------------------------------------
-#
-# standard python modules
-#
-#-------------------------------------------------------------------------
-from warnings import warn
+__revision__ = "$Revision: 7979 $"
 
 #-------------------------------------------------------------------------
 #
@@ -47,7 +42,6 @@ from _UrlBase import UrlBase
 from _Name import Name
 from _EventRef import EventRef
 from _PersonRef import PersonRef
-from _MarkerType import MarkerType
 from _AttributeType import AttributeType
 from _EventRoleType import EventRoleType
 
@@ -56,8 +50,8 @@ from _EventRoleType import EventRoleType
 # Person class
 #
 #-------------------------------------------------------------------------
-class Person(PrimaryObject,SourceBase,NoteBase,MediaBase,
-             AttributeBase,AddressBase,UrlBase,LdsOrdBase):
+class Person(SourceBase, NoteBase, AttributeBase, MediaBase,
+             AddressBase, UrlBase, LdsOrdBase, PrimaryObject):
     """
     Introduction
     ============
@@ -165,29 +159,28 @@ class Person(PrimaryObject,SourceBase,NoteBase,MediaBase,
             Person object
         @type data: tuple
         """
-        (
-            self.handle,             #  0
-            self.gramps_id,          #  1
-            self.gender,             #  2
-            primary_name,            #  3
-            alternate_names,         #  4
-            self.death_ref_index,    #  5
-            self.birth_ref_index,    #  6
-            event_ref_list,          #  7
-            self.family_list,        #  8
-            self.parent_family_list, #  9
-            media_list,              # 10
-            address_list,            # 11
-            attribute_list,          # 12
-            urls,                    # 13
-            lds_ord_list,            # 14
-            source_list,             # 15
-            note,                    # 16
-            self.change,             # 17
-            marker,                  # 18
-            self.private,            # 19
-            person_ref_list,         # 20
-            ) = data
+        (self.handle,             #  0
+         self.gramps_id,          #  1
+         self.gender,             #  2
+         primary_name,            #  3
+         alternate_names,         #  4
+         self.death_ref_index,    #  5
+         self.birth_ref_index,    #  6
+         event_ref_list,          #  7
+         self.family_list,        #  8
+         self.parent_family_list, #  9
+         media_list,              # 10
+         address_list,            # 11
+         attribute_list,          # 12
+         urls,                    # 13
+         lds_ord_list,            # 14
+         source_list,             # 15
+         note,                    # 16
+         self.change,             # 17
+         marker,                  # 18
+         self.private,            # 19
+         person_ref_list,         # 20
+         ) = data
 
         self.marker.unserialize(marker)
         self.primary_name.unserialize(primary_name)
@@ -206,6 +199,17 @@ class Person(PrimaryObject,SourceBase,NoteBase,MediaBase,
         NoteBase.unserialize(self, note)
             
     def _has_handle_reference(self, classname, handle):
+        """
+        Returns True if the object has reference to a given handle
+        of given primary object type.
+        
+        @param classname: The name of the primary object class.
+        @type classname: str
+        @param handle: The handle to be checked.
+        @type handle: str
+        @return: Returns whether the object has reference to this handle of this object type.
+        @rtype: bool
+        """
         if classname == 'Event':
             return handle in [ref.ref for ref in self.event_ref_list]
         elif classname == 'Person':
@@ -334,13 +338,6 @@ class Person(PrimaryObject,SourceBase,NoteBase,MediaBase,
         return self.get_sourcref_child_list() + self.source_list \
                + self.event_ref_list
 
-    def get_complete_flag(self):
-        warn( "Use get_marker instead of get_complete_flag",
-              DeprecationWarning, 2)
-        # Wrapper for old API
-        # remove when transitition done.
-        return self.marker == MarkerType.COMPLETE
-
     def set_primary_name(self, name):
         """
         Sets the primary name of the Person to the specified
@@ -648,6 +645,17 @@ class Person(PrimaryObject,SourceBase,NoteBase,MediaBase,
         """
         return self.parent_family_list
 
+    def set_parent_family_handle_list(self, family_list):
+        """
+        Returns the list of L{Family} handles in which the person is a
+        child.
+
+        @return: Returns the list of handles corresponding to the
+        L{Family} records with which the person is a child.
+        @rtype: list
+        """
+        self.parent_family_list = family_list
+
     def add_parent_family_handle(self, family_handle):
         """
         Adds the L{Family} handle to the Person instance's list of
@@ -663,7 +671,7 @@ class Person(PrimaryObject,SourceBase,NoteBase,MediaBase,
             Person's L{Family} list.
         @type family_handle: str
         """
-        if type(family_handle) not in (str ,unicode ):
+        if type(family_handle) not in (str, unicode):
             raise ValueError("expecting handle")
         if family_handle not in self.parent_family_list:
             self.parent_family_list.append(family_handle)
@@ -730,7 +738,7 @@ class Person(PrimaryObject,SourceBase,NoteBase,MediaBase,
         else:
             return self.parent_family_list[0]
 
-    def add_person_ref(self,person_ref):
+    def add_person_ref(self, person_ref):
         """
         Adds the L{PersonRef} to the Person instance's L{PersonRef} list.
         
