@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: _ReportUtils.py 7669 2006-11-22 22:30:47Z loshawlos $
+# $Id: _ReportUtils.py 7991 2007-01-26 14:55:11Z pez4brian $
 
 """
 A collection of utilities to aid in the generation of reports.
@@ -849,7 +849,7 @@ def cm2pt(cm):
     @returns: equivalent units in points
     @rtype: float
     """
-    return cm*182.88
+    return cm*28.3465
 
 def rgb_color(color):
     """
@@ -2285,3 +2285,32 @@ def get_person_mark(db,person):
         key = "%s (%s - %s)" % (name,birth,death)
         
     return BaseDoc.IndexMark( key, BaseDoc.INDEX_TYPE_ALP )
+
+#-------------------------------------------------------------------------
+#
+# Address String
+#
+#-------------------------------------------------------------------------
+def get_address_str(addr):
+    """
+    Returns a string that combines the elements of an addres
+    
+    @param addr: the GRAMPS address instance
+    """
+    str = ""
+    elems = [ addr.get_street(),
+              addr.get_city(),
+              addr.get_county(),
+              addr.get_state(),
+              addr.get_country(),
+              addr.get_postal_code(),
+              addr.get_phone()   ]
+    
+    for info in elems:
+        if info:
+            if str == "":
+                str = info
+            else:
+                str = "%s, %s" % (str,info)
+    return str
+    
