@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: Utils.py 7663 2006-11-21 16:20:35Z dallingham $
+# $Id: Utils.py 7997 2007-01-27 23:08:08Z dallingham $
 
 #-------------------------------------------------------------------------
 #
@@ -110,9 +110,10 @@ def fix_encoding(value):
         try:
             return unicode(value)
         except:
-            codeset = locale.getpreferredencoding()
-            if codeset == 'UTF-8':
-                codeset = 'latin1'
+            try:
+                codeset = locale.getpreferredencoding()
+            except:
+                codeset = "UTF-8"
             return unicode(value,codeset)
     else:
         return value
@@ -291,9 +292,16 @@ def find_file( filename):
         pass
     
     # Build list of alternate encodings
-    encodings = [sys.getfilesystemencoding(), locale.getpreferredencoding(),
-                 'UTF-8', 'ISO-8859-1']
-    encodings = list(set(encodings))
+    encodings = set()
+
+    for enc in [sys.getfilesystemencoding, locale.getpreferredencoding]:
+        try:
+            encodings.add(enc)
+        except:
+            pass
+    encodings.add('UTF-8')
+    encodings.add('ISO-8859-1')
+
     for enc in encodings:
         try:
             fname = filename.encode(enc)
@@ -608,9 +616,42 @@ def probably_alive(person,db,current_year=None,limit=0):
         if death_year > current_year + 110:
             # person died more tha 110 after current year
             return False
-            
-    # Neither birth nor death events are available.  Try looking
-    # for descendants that were born more than a lifespan ago.
+
+    # Neither birth nor death events are available. Try looking
+    # at siblings. If a sibling was born more than 120 years past,
+    # or more than 20 future, then problem then this person is
+    # probably not alive. If the sibling died more than 120 years
+    # past, or more than 120 years future, then probably not alive.
+
+    family_list = person.get_parent_family_handle_list()
+    for family_handle in family_list:
+        family = db.get_family_from_handle(family_handle)
+        for child_ref in family.get_child_ref_list():
+            child_handle = child_ref.ref
+            child = db.get_person_from_handle(child_handle)
+            child_birth_ref = child.get_birth_ref()
+            if child_birth_ref:
+                child_birth = db.get_event_from_handle(child_birth_ref.ref)
+                dobj = child_birth.get_date_object()
+                if dobj.get_start_date() != RelLib.Date.EMPTY:
+                    # if sibling birth date too far away, then not alive:
+                    year = dobj.get_year()
+                    if year != 0:
+                        if not (current_year - 120 < year < current_year + 20):
+                            return False
+            child_death_ref = child.get_death_ref()
+            if child_death_ref:
+                child_death = db.get_event_from_handle(child_death_ref.ref)
+                dobj = child_death.get_date_object()
+                if dobj.get_start_date() != RelLib.Date.EMPTY:
+                    # if sibling death date too far away, then not alive:
+                    year = dobj.get_year()
+                    if year != 0:
+                        if not (current_year - 120 < year < current_year + 120):
+                            return False
+
+    # Try looking for descendants that were born more than a lifespan
+    # ago.
 
     min_generation = 13
 
@@ -765,39 +806,34 @@ def get_source_referents(source_handle,db):
     secondary child-objects) to a given source handle in a given database.
     """
 
-    # Persons
-    person_list = [ handle \
-            for handle in db.get_person_handles(sort_handles=False) \
-            if db.get_person_from_handle(handle).has_source_reference(source_handle)
-    ]
+    # Use one pass through the reference map to grab all the references
+    object_list = [item for item in db.find_backlink_handles(source_handle)]
 
+    # Then form the object-specific lists
+
+    # Persons
+    person_list = [item[1] for item in object_list if item[0] == 'Person']
+    
     # Families
-    family_list = [ handle for handle in db.get_family_handles() \
-            if db.get_family_from_handle(handle).has_source_reference(source_handle)
-    ]
+    family_list = [item[1] for item in object_list if item[0] == 'Family']
 
     # Events
-    event_list = [ handle for handle in db.get_event_handles() \
-            if db.get_event_from_handle(handle).has_source_reference(source_handle)
-    ]
+    event_list = [item[1] for item in object_list if item[0] == 'Event']
 
     # Places
-    place_list = [ handle for handle in db.get_place_handles() \
-            if db.get_place_from_handle(handle).has_source_reference(source_handle)
-    ]
+    place_list = [item[1] for item in object_list if item[0] == 'Place']
 
     # Sources
-    source_list = [ handle for handle in db.get_source_handles() \
-            if db.get_source_from_handle(handle).has_source_reference(source_handle)
-    ]
+    source_list = [item[1] for item in object_list if item[0] == 'Source']
 
     # Media Objects
-    media_list = [ handle for handle in db.get_media_object_handles() \
-            if db.get_object_from_handle(handle).has_source_reference(source_handle)
-    ]
+    media_list = [item[1] for item in object_list if item[0] == 'MediaObject']
 
-    return (person_list,family_list,event_list,
-                place_list,source_list,media_list)
+    # Repositories
+    repo_list = [item[1] for item in object_list if item[0] == 'Repository']
+
+    return (person_list,family_list,event_list,place_list,source_list,
+            media_list,repo_list)
 
 def get_media_referents(media_handle,db):
     """
@@ -807,45 +843,28 @@ def get_media_referents(media_handle,db):
     to a given media handle in a given database.
     """
 
+    # Use one pass through the reference map to grab all the references
+    object_list = [item for item in db.find_backlink_handles(media_handle)]
+
+    # Then form the object-specific lists
+
     # Persons
-    person_list = [ handle \
-            for handle in db.get_person_handles(sort_handles=False) \
-            if media_handle in \
-                    [photo.get_reference_handle() for photo \
-                    in db.get_person_from_handle(handle).get_media_list()]
-    ]
+    person_list = [item[1] for item in object_list if item[0] == 'Person']
 
     # Families
-    family_list = [ handle for handle in db.get_family_handles() \
-            if media_handle in \
-                    [photo.get_reference_handle() for photo \
-                    in db.get_family_from_handle(handle).get_media_list()]
-    ]
+    family_list = [item[1] for item in object_list if item[0] == 'Family']
 
     # Events
-    event_list = [ handle for handle in db.get_event_handles() \
-            if media_handle in \
-                    [photo.get_reference_handle() for photo \
-                    in db.get_event_from_handle(handle).get_media_list()]
-    ]
+    event_list = [ item[1] for item in object_list if item[0] == 'Event']
 
     # Places
-    place_list = [ handle for handle in db.get_place_handles() \
-            if media_handle in \
-                    [photo.get_reference_handle() for photo \
-                    in db.get_place_from_handle(handle).get_media_list()]
-    ]
+    place_list = [ item[1] for item in object_list if item[0] == 'Place']
 
     # Sources
-    source_list = [ handle for handle in db.get_source_handles() \
-            if media_handle in \
-                    [photo.get_reference_handle() for photo \
-                    in db.get_source_from_handle(handle).get_media_list()]
-    ]
+    source_list = [ item[1] for item in object_list if item[0] == 'Source']
 
     return (person_list,family_list,event_list,place_list,source_list)
 
-
 #-------------------------------------------------------------------------
 #
 # 
@@ -1043,3 +1062,18 @@ def launch(prog_str,path):
     
         os.spawnvpe(os.P_NOWAIT, prog, prog_list, os.environ)
 
+def profile(func,*args):
+    import hotshot, hotshot.stats
+
+    pr = hotshot.Profile('mystats.profile')
+    print "Start"
+    pr.runcall(func,*args)
+    print "Finished"
+    pr.close()
+    print "Loading profile"
+    stats = hotshot.stats.load('mystats.profile')
+    print "done"
+    stats.strip_dirs()
+    stats.sort_stats('time','calls')
+    stats.print_stats(100)
+    
