@@ -18,14 +18,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ViewManager.py 7653 2006-11-20 02:08:42Z pez4brian $
+# $Id: ViewManager.py 8004 2007-01-28 18:20:10Z rshura $
 
 """
 Manages the main window and the pluggable views
 """
 
 __author__ = "Donald N. Allingham"
-__revision__ = "$Revision: 7653 $"
+__revision__ = "$Revision: 8004 $"
 
 #-------------------------------------------------------------------------
 #
@@ -78,6 +78,7 @@ import GrampsWidgets
 import UndoHistory
 from DbLoader import DbLoader
 import GrampsDisplay
+
 def show_url(dialog,link,user_data):
     GrampsDisplay.url(link)
 gtk.about_dialog_set_url_hook(show_url,None)
@@ -145,6 +146,7 @@ uidefault = '''<ui>
   <menu action="HelpMenu">
     <menuitem action="UserManual"/>
     <menuitem action="FAQ"/>
+    <menuitem action="KeyBindings"/>
     <menuitem action="TipOfDay"/>
     <menuitem action="PluginStatus"/>
     <separator/>
@@ -176,6 +178,10 @@ uidefault = '''<ui>
 <accelerator action="F9"/>
 <accelerator action="F11"/>
 <accelerator action="F12"/>
+<accelerator action="<CONTROL>BackSpace"/>
+<accelerator action="<CONTROL>J"/>
+<accelerator action="<Alt>N"/>
+<accelerator action="<Alt>P"/>
 </ui>
 '''
 
@@ -335,33 +341,24 @@ class ViewManager:
             ('PluginStatus', None,_('_Plugin status'), None, None,
              self.plugin_status), 
             ('FAQ', None, _('_FAQ'), None, None, self.faq_activate), 
+            ('KeyBindings', None, _('_Key Bindings'), None, None, self.key_bindings), 
             ('UserManual', gtk.STOCK_HELP, _('_User Manual'), 'F1', None,
              self.manual_activate), 
             ('TipOfDay', None, _('Tip of the day'), None, None,
              self.tip_of_day_activate), 
             ]
 
-        self._action_action_list = [
+        self._readonly_action_list = [
             ('SaveAs', gtk.STOCK_SAVE_AS, _('_Save As'),"<control><shift>s",
              None, self.save_as_activate), 
             ('Export', 'gramps-export', _('_Export'), "<control>e", None,
              self.export_data), 
             ('Abandon', gtk.STOCK_REVERT_TO_SAVED,
              _('_Abandon changes and quit'), None, None, self.abort), 
-            ('ScratchPad', gtk.STOCK_PASTE, _('_ScratchPad'), "",
-             _("Open the ScratchPad dialog"), self.scratchpad), 
-            ('Import', gtk.STOCK_CONVERT, _('_Import'), "<control>i", None,
-             self.import_data), 
             ('Reports', 'gramps-reports', _('_Reports'), None,
              _("Open the reports dialog"), self.reports_clicked), 
-            ('Tools', 'gramps-tools', _('_Tools'), None,
-             _("Open the tools dialog"), self.tools_clicked),
-            ('EditMenu', None, _('_Edit')), 
-            ('ColumnEdit', gtk.STOCK_PROPERTIES, _('_Column Editor')), 
             ('GoMenu', None, _('_Go')), 
-            ('BookMenu', None, _('_Bookmarks')), 
             ('ReportsMenu', None, _('_Reports')), 
-            ('ToolsMenu', None, _('_Tools')), 
             ('WindowsMenu', None, _('_Windows')),
             ('F2', None, 'F2', "F2", None, self.keypress),
             ('F3', None, 'F3', "F3", None, self.keypress),
@@ -372,7 +369,26 @@ class ViewManager:
             ('F8', None, 'F9', "F8", None, self.keypress),
             ('F9', None, 'F9', "F9", None, self.keypress),
             ('F11', None, 'F11', "F11", None, self.keypress),
+            ('<CONTROL>BackSpace', None, '<CONTROL>BackSpace', "<CONTROL>BackSpace", None, self.keypress),
+            ('<CONTROL>Delete', None, '<CONTROL>Delete', "<CONTROL>Delete", None, self.keypress),
+            ('<CONTROL>Insert', None, '<CONTROL>Insert', "<CONTROL>Insert", None, self.keypress),
             ('F12', None, 'F12', "F12", None, self.keypress),
+            ('<CONTROL>J', None, '<CONTROL>J', "<CONTROL>J", None, self.keypress),
+            ('<Alt>N', None, '<Alt>N', "<Alt>N", None, self.next_view),
+            ('<Alt>P', None, '<Alt>P', "<Alt>P", None, self.prev_view),
+            ]
+
+        self._action_action_list = [
+            ('ScratchPad', gtk.STOCK_PASTE, _('_ScratchPad'), "<alt>s",
+             _("Open the ScratchPad dialog"), self.scratchpad), 
+            ('Import', gtk.STOCK_CONVERT, _('_Import'), "<control>i", None,
+             self.import_data), 
+            ('Tools', 'gramps-tools', _('_Tools'), None,
+             _("Open the tools dialog"), self.tools_clicked),
+            ('EditMenu', None, _('_Edit')), 
+            ('ColumnEdit', gtk.STOCK_PROPERTIES, _('_Column Editor')), 
+            ('BookMenu', None, _('_Bookmarks')), 
+            ('ToolsMenu', None, _('_Tools')), 
             ]
 
         self._file_toggle_action_list = [
@@ -395,7 +411,7 @@ class ViewManager:
 
         self._undo_history_action_list = [
             ('UndoHistory', 'gramps-undo-history',
-             _('Undo History'), None, None, self.undo_history),
+             _('Undo History'), "<control>H", None, self.undo_history),
             ]
 
         self._navigation_type = {
@@ -411,15 +427,34 @@ class ViewManager:
             self.uistate.push_message(self.state,
                                       _("Key %s is not bound") % name)
 
+    def next_view(self, action):
+        current_page = self.notebook.get_current_page()
+        if current_page == len(self.pages)-1:
+            new_page = 0
+        else:
+            new_page = current_page + 1
+        self.buttons[new_page].set_active(True)
+
+    def prev_view(self, action):
+        current_page = self.notebook.get_current_page()
+        if current_page == 0:
+            new_page = len(self.pages)-1
+        else:
+            new_page = current_page - 1
+        self.buttons[new_page].set_active(True)
+
     def init_interface(self):
         self._init_lists()
 
         self.create_pages()
         if not self.file_loaded:
             self.actiongroup.set_visible(False)
+            self.readonlygroup.set_visible(False)
         self.fileactions.set_sensitive(False)
-        self.build_tools_menu()
-        self.build_report_menu()
+        self.build_tools_menu(tool_list)
+        self.build_report_menu(report_list)
+        self.uistate.connect('plugins-reloaded',
+                             self.rebuild_report_and_tool_menus)
         self.fileactions.set_sensitive(True)
         self.uistate.widget.set_sensitive(True)
         Config.client.notify_add("/apps/gramps/interface/statusbar",
@@ -452,6 +487,7 @@ class ViewManager:
         self.uistate.push_message(self.state,_('Ready'))
 
     def quit(self, *obj):
+        self.uistate.set_sensitive(False)
         self.backup()
         self.state.db.close()
         (width, height) = self.window.get_size()
@@ -525,6 +561,7 @@ class ViewManager:
         self.window.add_accel_group(accelgroup)
 
         self.actiongroup = gtk.ActionGroup('MainWindow')
+        self.readonlygroup = gtk.ActionGroup('AllMainWindow')
         self.fileactions = gtk.ActionGroup('FileWindow')
         self.undoactions = gtk.ActionGroup('Undo')
         self.redoactions = gtk.ActionGroup('Redo')
@@ -532,6 +569,7 @@ class ViewManager:
 
         self.fileactions.add_actions(self._file_action_list)
         self.actiongroup.add_actions(self._action_action_list)
+        self.readonlygroup.add_actions(self._readonly_action_list)
         self.fileactions.add_toggle_actions(self._file_toggle_action_list)
 
         self.undoactions.add_actions(self._undo_action_list)
@@ -546,6 +584,7 @@ class ViewManager:
         
         self.uimanager.insert_action_group(self.fileactions, 1)
         self.uimanager.insert_action_group(self.actiongroup, 1)
+        self.uimanager.insert_action_group(self.readonlygroup, 1)
         self.uimanager.insert_action_group(self.undoactions, 1)
         self.uimanager.insert_action_group(self.redoactions, 1)
         self.uimanager.insert_action_group(self.undohistoryactions, 1)
@@ -587,6 +626,14 @@ class ViewManager:
         except gobject.GError, msg:
             QuestionDialog.ErrorDialog(_("Could not open help"), str(msg))
 
+    def key_bindings(self, obj):
+        """Display FAQ"""
+        try:
+            import GrampsDisplay
+            GrampsDisplay.help('keybind-lists')
+        except gobject.GError, msg:
+            QuestionDialog.ErrorDialog(_("Could not open help"), str(msg))
+
     def tip_of_day_activate(self, obj):
         """Display Tip of the day"""
         TipOfDay.TipOfDay(self.uistate)
@@ -617,7 +664,12 @@ class ViewManager:
         about.set_website_label(_('GRAMPS Homepage'))
         about.set_website(const.url_homepage)
         about.set_authors(const.authors)
-        about.set_translator_credits(_(const.translators))
+
+        # Only set translation credits if they are translated
+        trans_credits = _(const.translators)
+        if trans_credits != const.translators:
+            about.set_translator_credits(trans_credits)
+
         about.set_documenters(const.documenters)
         about.set_logo(gtk.gdk.pixbuf_new_from_file(const.splash))
         about.set_modal(True)
@@ -652,6 +704,12 @@ class ViewManager:
     def register_view(self, view):
         self.views.append(view)
 
+    def switch_page_on_dnd(self, widget, context, x, y, time, page_no):
+        self.vb_handlers_block()
+        if self.notebook.get_current_page() != page_no:
+            self.notebook.set_current_page(page_no)
+        self.vb_handlers_unblock()
+    
     def create_pages(self):
         self.pages = []
         self.prev_nav = PageView.NAVIGATION_NONE
@@ -677,8 +735,12 @@ class ViewManager:
             page_display = page.get_display()
             page_display.show_all()
             page.post()
-            self.notebook.append_page(page_display, hbox)
+            page_no = self.notebook.append_page(page_display, hbox)
             self.pages.append(page)
+            
+            # Enable view switching during DnD
+            hbox.drag_dest_set(0, [], 0)
+            hbox.connect('drag_motion', self.switch_page_on_dnd,page_no)
 
             # create the button and add it to the sidebar
             button = gtk.ToggleButton()
@@ -707,27 +769,35 @@ class ViewManager:
             self.bbox.pack_start(button, False)
             self.buttons.append(button)
             self.button_handlers.append(handler_id)
+            
+            # Enable view switching during DnD
+            button.drag_dest_set(0, [], 0)
+            button.connect('drag_motion', self.switch_page_on_dnd,page_no)
+
 
         use_current = Config.get(Config.USE_LAST_VIEW)
         if use_current:
-            current = Config.get(Config.LAST_VIEW)
-            if current > len(self.pages):
-                current = 0
+            current_page = Config.get(Config.LAST_VIEW)
+            if current_page > len(self.pages):
+                current_page = 0
         else:
-            current = 0
+            current_page = 0
 
-        self.active_page = self.pages[current]
-        self.buttons[current].set_active(True)
+        self.active_page = self.pages[current_page]
+        self.buttons[current_page].set_active(True)
         self.active_page.set_active()
-        self.notebook.set_current_page(current)
+        self.notebook.set_current_page(current_page)
 
     def vb_clicked(self,button,index):
         if Config.get(Config.VIEW):
             self.vb_handlers_block()
             self.notebook.set_current_page(index)
-            # If the click is on the same view we're in,
-            # restore the button state to active
-            if not button.get_active():
+            # FIXME: This used to work, but now DnD switches views
+            # and messes this up
+
+            # If the click is on the same view we're in, 	 
+            # restore the button state to active 	 
+            if not button.get_active(): 	 
                 button.set_active(True)
             self.vb_handlers_unblock()
 
@@ -748,9 +818,12 @@ class ViewManager:
         if num == -1:
             num = self.notebook.get_current_page()
 
+        # set button of current page active
         for ix in range(len(self.buttons)):
             if ix != num and self.buttons[ix].get_active():
                 self.buttons[ix].set_active(False)
+            if ix == num and not self.buttons[ix].get_active():
+                self.buttons[ix].set_active(True)
 
         if self.state.open:
 
@@ -802,6 +875,11 @@ class ViewManager:
 
         self.page_is_changing = False
 
+    def import_pkg(self, filename):
+	import ReadPkg
+	ReadPkg.impData(self.state.db, filename, self.uistate.pulse_progressbar)
+	self.post_load()
+
     def import_data(self, obj):
         if self.state.db.db_is_open:
             self.db_loader.import_file()
@@ -858,9 +936,11 @@ class ViewManager:
         if self.state.db.readonly:
             msg =  "%s (%s) - GRAMPS" % (name, _('Read Only'))
             self.uistate.window.set_title(msg)
+            self.actiongroup.set_sensitive(False)
         else:
             msg = "%s - GRAMPS" % name
             self.uistate.window.set_title(msg)
+            self.actiongroup.set_sensitive(True)
 
         res = self.state.db.get_researcher()
         owner = GrampsCfg.get_researcher()
@@ -876,7 +956,6 @@ class ViewManager:
 
         self.state.db.enable_signals()
         self.state.signal_change()
-        self.state.db.request_rebuild()
 
         Config.set(Config.RECENT_FILE,filename)
     
@@ -889,6 +968,7 @@ class ViewManager:
         
         self.change_page(None, None)
         self.actiongroup.set_visible(True)
+	self.readonlygroup.set_visible(True)
 
         self.file_loaded = True
 
@@ -986,10 +1066,14 @@ class ViewManager:
             pass            
 
     def undo(self, obj):
+        self.uistate.set_busy_cursor(1)
         self.state.db.undo()
+        self.uistate.set_busy_cursor(0)
 
     def redo(self, obj):
+        self.uistate.set_busy_cursor(1)
         self.state.db.redo()
+        self.uistate.set_busy_cursor(0)
 
     def undo_history(self, obj):
         try:
@@ -1003,23 +1087,27 @@ class ViewManager:
             import Exporter
             Exporter.Exporter(self.state, self.uistate)
 
-    def build_tools_menu(self):
+    def rebuild_report_and_tool_menus(self,tool_list,report_list):
+        self.build_tools_menu(tool_list)
+        self.build_report_menu(report_list)
+
+    def build_tools_menu(self,tool_list):
         self.toolactions = gtk.ActionGroup('ToolWindow')
         (ui, actions) = self.build_plugin_menu('ToolsMenu', 
-                                              tool_list, 
-                                              Tool.tool_categories, 
-                                              make_tool_callback)
+                                               tool_list, 
+                                               Tool.tool_categories, 
+                                               make_tool_callback)
         self.toolactions.add_actions(actions)
         self.uistate.uimanager.add_ui_from_string(ui)
         self.uimanager.insert_action_group(self.toolactions, 1)
         self.uistate.uimanager.ensure_update()
     
-    def build_report_menu(self):
+    def build_report_menu(self,report_list):
         self.reportactions = gtk.ActionGroup('ReportWindow')
         (ui, actions) = self.build_plugin_menu('ReportsMenu', 
-                                              report_list, 
-                                              standalone_categories, 
-                                              make_report_callback)
+                                               report_list, 
+                                               standalone_categories, 
+                                               make_report_callback)
         self.reportactions.add_actions(actions)
         self.uistate.uimanager.add_ui_from_string(ui)
         self.uimanager.insert_action_group(self.reportactions, 1)
@@ -1115,20 +1203,7 @@ def check_for_portability_problems(filetype):
         if version < (2, 5) and not Config.get(Config.PORT_WARN):
             QuestionDialog.MessageHideDialog(
                 _('Database is not portable'),
-                _('Your system is running an old version of python. This '
-                  'prevents you from being able to copy your database to '
-                  'other machines. For most people, this is not a problem.\n\n'
-                  'If you need to transfer the database to another machine, '
+                _('If you need to transfer the database to another machine, '
                   'export to a GRAMPS Package, and import the GRAMPS Package '
-                  'on the other machine.\n\nIf you feel you need to be able '
-                  'to transfer this file between machines without exporting, '
-                  'you need to either upgrade to version 2.5 of python, or '
-                  'disable transactions in the preferences menu. Disabling '
-                  'transactions will slow down your performance, and may '
-                  'allow your database to become corrupted if an error '
-                  'occurs while data is being saved'),
+                  'on the other machine.'),
                 Config.PORT_WARN)
-                 
-                 
-                
-    
