@@ -20,7 +20,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: const.py.in 7697 2006-11-26 03:00:44Z rshura $
+# $Id: const.py.in 7889 2007-01-10 02:14:25Z rshura $
 
 #-------------------------------------------------------------------------
 #
@@ -63,7 +63,8 @@ convert = "convert"
 pixdir = "/usr/share/pixmaps"
 nautdir = "/usr/share/pixmaps/nautilus"
 
-prefixdir = "@GPREFIX@"
+prefixdir = "/usr/local"
+sysconfdir = "${prefix}/etc"
 
 #-------------------------------------------------------------------------
 #
@@ -143,7 +144,7 @@ else:
 #
 #-------------------------------------------------------------------------
 program_name   = "GRAMPS"
-version        = "2.2.3-1"
+version        = "2.2.6-1"
 copyright_msg  = unicode(" 2001-2006 Donald N. Allingham","iso-8859-1")
 comments       = _("GRAMPS (Genealogical Research and Analysis "
                    "Management Programming System) is a personal "
