@@ -1,7 +1,7 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2000-2005  Donald N. Allingham
+# Copyright (C) 2000-2006  Donald N. Allingham
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: HtmlDoc.py,v 1.34.2.1 2005/11/25 03:09:10 rshura Exp $
+# $Id: HtmlDoc.py 6911 2006-06-18 20:58:25Z pez4brian $
 
 #------------------------------------------------------------------------
 #
@@ -35,14 +35,16 @@ from gettext import gettext as _
 # GRAMPS modules 
 #
 #------------------------------------------------------------------------
-import PluginMgr
+from PluginUtils import register_text_doc
 import ImgManip
-import TarFile
+import tarfile
 import const
 import Errors
 import BaseDoc
 import QuestionDialog
-import GrampsMime
+import Mime
+import Utils
+
 
 #------------------------------------------------------------------------
 #
@@ -148,8 +150,10 @@ class HtmlDoc(BaseDoc.BaseDoc):
         stop = re.compile(r"<!--\s*STOP\s*-->")
         top_add = 1
         bottom_add = 0
-        tf = TarFile.ReadTarFile(self.template,None)
-        self.map = tf.extract_files()
+        archive = tarfile.open(self.template)
+        for tarinfo in archive:
+            self.map[tarinfo.name] = archive.extractfile(tarinfo)
+        archive.close()
         templateFile = self.map['template.html']
         while 1:
             line = templateFile.readline()
@@ -354,9 +358,8 @@ class HtmlDoc(BaseDoc.BaseDoc):
 
         if self.print_req:
             apptype = 'text/html'
-            app = GrampsMime.get_application(apptype)
-            os.environ["FILE"] = self.filename
-            os.system ('%s "$FILE" &' % app[0])
+            app = Mime.get_application(apptype)
+            Utils.launch(app[0],self.filename)
 
     def write_support_files(self):
         if self.map:
@@ -478,7 +481,7 @@ class HtmlDoc(BaseDoc.BaseDoc):
                 self.write_text(line.strip().replace('\n',' '))
                 self.end_paragraph()
 
-    def write_text(self,text):
+    def write_text(self,text,mark=None):
         text = text.replace('&','&amp;');       # Must be first
         text = text.replace('<','&lt;');
         text = text.replace('>','&gt;');
@@ -494,16 +497,19 @@ class HtmlDoc(BaseDoc.BaseDoc):
 # Register the document generator with the GRAMPS plugin system
 #
 #------------------------------------------------------------------------
+print_label = None
 try:
-    import Utils
-
-    prog = GrampsMime.get_application("text/html")
-    mtype = GrampsMime.get_description("text/html")
+    prog = Mime.get_application("text/html")
+    mtype = Mime.get_description("text/html")
     
     if Utils.search_for(prog[0]):
         print_label=_("Open in %s") % prog[1]
     else:
         print_label=None
-    PluginMgr.register_text_doc(mtype,HtmlDoc,1,0,1,".html", print_label)
+        
+    if mtype == _("unknown"):
+        mtype = _('HTML')
+        
+    register_text_doc(mtype,HtmlDoc,1,0,1,".html", print_label)
 except:
-    PluginMgr.register_text_doc(_('HTML'),HtmlDoc,1,0,1,".html", None)
+    register_text_doc(_('HTML'),HtmlDoc,1,0,1,".html", None)
