@@ -1,7 +1,7 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2000-2005  Donald N. Allingham
+# Copyright (C) 2000-2006  Donald N. Allingham
 #
 # Modifications and feature additions:
 #               2002-2003  Donald A. Peterson
@@ -24,7 +24,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: LaTeXDoc.py,v 1.18.2.1 2005/12/02 01:15:20 rshura Exp $
+# $Id: LaTeXDoc.py 6850 2006-06-03 03:32:26Z pez4brian $
 
 """LaTeX document generator"""
 
@@ -41,7 +41,7 @@ from gettext import gettext as _
 #
 #------------------------------------------------------------------------
 import BaseDoc
-import PluginMgr
+from PluginUtils import register_text_doc
 import ImgManip
 import Errors
 
@@ -484,7 +484,7 @@ class LaTeXDoc(BaseDoc.BaseDoc):
             self.f.write('\\end{verbatim}')
         self.end_paragraph()
 
-    def write_text(self,text):
+    def write_text(self,text,mark=None):
         """Write the text to the file"""
 	if text == '\n':
 	    text = '\\newline\n'
@@ -501,7 +501,7 @@ class LaTeXDoc(BaseDoc.BaseDoc):
 # Register the document generator with the system
 #
 #------------------------------------------------------------------------
-PluginMgr.register_text_doc(
+register_text_doc(
     name=_("LaTeX"),
     classref=LaTeXDoc,
     table=1,
