@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ODFDoc.py 7706 2006-11-26 19:33:48Z dallingham $
+# $Id: ODFDoc.py 7977 2007-01-25 03:26:00Z pez4brian $
 
 #-------------------------------------------------------------------------
 #
@@ -1185,5 +1185,5 @@ except:
     print_label = None
 
 register_text_doc(_('Open Document Text'), ODFDoc, 1, 1, 1, ".odt", print_label)
-register_book_doc(_("Open Document Text"), ODFDoc, 1, 1, 1, ".odt")
+register_book_doc(_("Open Document Text"), ODFDoc, 1, 1, 1, ".odt", print_label)
 register_draw_doc(_("Open Document Text"), ODFDoc, 1, 1, ".odt", print_label);
