@@ -1,7 +1,7 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2000-2005  Donald N. Allingham
+# Copyright (C) 2000-2006  Donald N. Allingham
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: AncestorChart2.py,v 1.24.2.4 2005/11/11 04:53:58 dallingham Exp $
+# $Id: AncestorChart2.py 6963 2006-06-25 01:57:56Z pez4brian $
 
 "Graphical Reports/Ancestor Chart"
 
@@ -43,10 +43,13 @@ import gtk
 #
 #------------------------------------------------------------------------
 import BaseDoc
-import Report
 from SubstKeywords import SubstKeywords
-from ReportUtils import pt2cm, cm2pt
-import ReportOptions
+from PluginUtils import register_report
+from ReportBase import Report, ReportUtils, ReportOptions, \
+     CATEGORY_DRAW, MODE_GUI, MODE_BKI, MODE_CLI
+import NameDisplay
+pt2cm = ReportUtils.pt2cm
+cm2pt = ReportUtils.cm2pt
 
 #------------------------------------------------------------------------
 #
@@ -148,7 +151,6 @@ class GenChart:
     def display(self):
         index = 0
         for i in self.array:
-            print "%04d" % index,i
             index=index+1
 
     def not_blank(self,line):
@@ -162,7 +164,7 @@ class GenChart:
 # AncestorChart
 #
 #------------------------------------------------------------------------
-class AncestorChart(Report.Report):
+class AncestorChart(Report):
 
     def __init__(self,database,person,options_class):
         """
@@ -182,16 +184,17 @@ class AncestorChart(Report.Report):
         dispf     - Display format for the output box.
         singlep   - Whether to scale to fit on a single page.
         compress  - Whether to compress chart.
-        title     - Title of the report displayed on top.
         """
-        Report.Report.__init__(self,database,person,options_class)
+        Report.__init__(self,database,person,options_class)
 
         (self.max_generations,self.pgbrk) \
                         = options_class.get_report_generations()
         self.display = options_class.handler.options_dict['dispf']
         self.force_fit = options_class.handler.options_dict['singlep']
         self.compress = options_class.handler.options_dict['compress']
-        self.title = options_class.handler.options_dict['title'].strip()
+        
+        name = NameDisplay.displayer.display_formal(person)
+        self.title = _("Ancestor Graph for %s") % name
 
         self.map = {}
         self.text = {}
@@ -418,21 +421,20 @@ class AncestorChart(Report.Report):
 # 
 #
 #------------------------------------------------------------------------
-class AncestorChartOptions(ReportOptions.ReportOptions):
+class AncestorChartOptions(ReportOptions):
 
     """
     Defines options and provides handling interface.
     """
 
     def __init__(self,name,person_id=None):
-        ReportOptions.ReportOptions.__init__(self,name,person_id)
+        ReportOptions.__init__(self,name,person_id)
 
     def set_new_options(self):
         # Options specific for this report
         self.options_dict = {
             'singlep'   : 1,
             'compress'  : 1,
-            'title'     : '',
         }
         self.options_help = {
             'singlep'   : ("=0/1","Whether to scale to fit on a single page.",
@@ -441,8 +443,6 @@ class AncestorChartOptions(ReportOptions.ReportOptions):
             'compress'  : ("=0/1","Whether to compress chart.",
                             ["Do not compress chart","Compress chart"],
                             True),
-            'title'     : ("=str","Title string for the report",
-                            "Whatever String You Wish"),
         }
 
     def enable_options(self):
@@ -473,20 +473,12 @@ class AncestorChartOptions(ReportOptions.ReportOptions):
         self.compress.set_active(self.options_dict['compress'])
         dialog.add_option('',self.compress)
 
-        self.title_box = gtk.Entry()
-        if self.options_dict['title']:
-            self.title_box.set_text(self.options_dict['title'])
-        else:
-            self.title_box.set_text(dialog.get_header(dialog.person.get_primary_name().get_name()))
-        dialog.add_option(_('Title'),self.title_box)
-
     def parse_user_options(self,dialog):
         """
         Parses the custom options that we have added.
         """
         self.options_dict['singlep'] = int(self.scale.get_active ())
         self.options_dict['compress'] = int(self.compress.get_active ())
-        self.options_dict['title'] = unicode(self.title_box.get_text()).strip()
 
     def make_default_style(self,default_style):
         """Make the default output style for the Ancestor Chart report."""
@@ -512,13 +504,12 @@ class AncestorChartOptions(ReportOptions.ReportOptions):
 # 
 #
 #------------------------------------------------------------------------
-from PluginMgr import register_report
 register_report(
     name = 'ancestor_chart2',
-    category = Report.CATEGORY_DRAW,
+    category = CATEGORY_DRAW,
     report_class = AncestorChart,
     options_class = AncestorChartOptions,
-    modes = Report.MODE_GUI | Report.MODE_BKI | Report.MODE_CLI,
+    modes = MODE_GUI | MODE_BKI | MODE_CLI,
     translated_name = _("Ancestor Graph"),
     status = _("Stable"),
     author_name = "Donald N. Allingham",
