@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: BookReport.py 7367 2006-10-08 02:38:51Z rshura $
+# $Id: BookReport.py 7976 2007-01-25 03:22:46Z pez4brian $
 
 # Written by Alex Roitman, 
 # largely based on the BaseDoc classes by Don Allingham
@@ -1067,6 +1067,9 @@ class BookReportDialog(ReportDialog):
             self.rptlist.append(obj)
             newpage = 1
         self.doc.open(self.target_path)
+        
+        if self.print_report.get_active():
+            self.doc.print_requested ()
 
     def make_report(self):
         """The actual book report. Start it out, then go through the item list 
