@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: Check.py 7714 2006-11-27 02:55:40Z rshura $
+# $Id: Check.py 7951 2007-01-22 03:56:23Z dallingham $
 
 "Database Processing/Check and repair database"
 
@@ -28,8 +28,13 @@
 #
 #-------------------------------------------------------------------------
 import os
+import sys
 import cStringIO
-import sets
+try:
+    set()
+except:
+    from sets import Set as set
+
 from gettext import gettext as _
 
 #------------------------------------------------------------------------
@@ -98,7 +103,7 @@ def _table_low_level(db,table):
     Low level repair for a given db table.
     """
     handle_list = table.keys()
-    dup_handles = sets.Set(
+    dup_handles = set(
         [ handle for handle in handle_list if handle_list.count(handle) > 1 ]
         )
 
@@ -183,6 +188,7 @@ class Check(Tool.BatchTool):
         checker.check_person_references()
         checker.check_place_references()
         checker.check_source_references()
+        checker.check_repo_references()
         self.db.transaction_commit(trans, _("Check Integrity"))
         self.db.enable_signals()
         self.db.request_rebuild()
@@ -215,6 +221,7 @@ class CheckIntegrity:
         self.invalid_person_references = []
         self.invalid_place_references = []
         self.invalid_source_references = []
+        self.invalid_repo_references = []
         self.removed_name_format = []
         self.progress = Utils.ProgressMeter(_('Checking database'),'')
 
@@ -288,7 +295,7 @@ class CheckIntegrity:
             value = self.db.person_map[handle]
             p = RelLib.Person(value)
             splist = p.get_family_handle_list()
-            if len(splist) != len(sets.Set(splist)):
+            if len(splist) != len(set(splist)):
                 new_list = []
                 for value in splist:
                     if value not in new_list:
@@ -309,6 +316,17 @@ class CheckIntegrity:
                 obj.path = Utils.fix_encoding( obj.path)
                 obj.desc = Utils.fix_encoding( obj.desc)
                 self.db.commit_media_object(obj,self.trans)
+            # Once we are here, fix the mime string if not str
+            if type(data[3]) != str:
+                obj = self.db.get_object_from_handle(handle)
+                try:
+                    if data[3] == str(data[3]):
+                        obj.mime = str(data[3])
+                    else:
+                        obj.mime = ""
+                except:
+                    obj.mime = ""
+                self.db.commit_media_object(obj,self.trans)
             self.progress.step()
 
     def check_for_broken_family_links(self):
@@ -375,11 +393,34 @@ class CheckIntegrity:
                     family.remove_child_ref(child_ref)
                     self.db.commit_family(family,self.trans)
                     self.broken_links.append((child_handle,family_handle))
+
+            new_ref_list = []
+            new_ref_handles = []
+            replace = False
+            for child_ref in family.get_child_ref_list():
+                child_handle = child_ref.ref
+                if child_handle in new_ref_handles:
+                    replace = True
+                else:
+                    new_ref_list.append(child_ref)
+                    new_ref_handles.append(child_handle)
+
+            if replace:
+                family.set_child_ref_list(new_ref_list)
+                self.db.commit_family(family,self.trans)
+
             self.progress.step()
             
         # Check persons membership in referenced families
         for person_handle in self.db.get_person_handles():
             person = self.db.get_person_from_handle(person_handle)
+
+            phandle_list = person.get_parent_family_handle_list()
+            new_list = list(set(phandle_list))
+            if len(phandle_list) != len(new_list):
+                person.set_parent_family_handle_list(new_list)
+                self.db.commit_person(person,self.trans)
+
             for par_family_handle in person.get_parent_family_handle_list():
                 family = self.db.get_family_from_handle(par_family_handle)
                 if not family:
@@ -465,7 +506,8 @@ class CheckIntegrity:
                 self.bad_photo.append(ObjectId)
 
             def fs_ok_clicked(obj):
-                name = fs_top.get_filename()
+                name = unicode(fs_top.get_filename(),
+                               sys.getfilesystemencoding())
                 if os.path.isfile(name):
                     obj = self.db.get_object_from_handle(ObjectId)
                     obj.set_path(name)
@@ -683,6 +725,22 @@ class CheckIntegrity:
                     self.db.commit_person(person,self.trans)
                     self.invalid_person_references.append(key)
 
+    def check_repo_references(self):
+        slist = self.db.get_source_handles()
+        
+        self.progress.set_pass(_('Looking for repository reference problems'),
+                               len(slist))
+        
+        for key in slist:
+            source = self.db.get_source_from_handle(key)
+            for reporef in source.get_reporef_list():
+                r = self.db.get_repository_from_handle(reporef.ref)
+                if not r:
+                    # The referenced repository does not exist in the database
+                    source.get_reporef_list().remove(reporef)
+                    self.db.commit_source(source,self.trans)
+                    self.invalid_repo_references.append(key)
+
     def check_place_references(self):
         plist = self.db.get_person_handles()
         flist = self.db.get_family_handles()
@@ -850,10 +908,12 @@ class CheckIntegrity:
         person_references = len(self.invalid_person_references)
         place_references = len(self.invalid_place_references)
         source_references = len(self.invalid_source_references)
+        repo_references = len(self.invalid_repo_references)
         name_format = len(self.removed_name_format)
 
         errors = blink + efam + photos + rel + person + event_invalid\
-                 + person_references + place_references + source_references
+                 + person_references + place_references + source_references\
+                 + repo_references
         
         if errors == 0:
             if cl:
@@ -933,7 +993,12 @@ class CheckIntegrity:
         if person_references == 1:
             self.text.write(_("1 person was referenced but not found\n"))
         elif person_references > 1:
-            self.text.write(_("%d person were referenced, but not found\n") % person_references)
+            self.text.write(_("%d persons were referenced, but not found\n") % person_references)
+        
+        if repo_references == 1:
+            self.text.write(_("1 repository was referenced but not found\n"))
+        elif repo_references > 1:
+            self.text.write(_("%d repositories were referenced, but not found\n") % repo_references)
         if photos == 1:
             self.text.write(_("1 media object was referenced, but not found\n"))
         elif photos > 1:
