@@ -2,7 +2,7 @@
 # count_anc.py - Ancestor counting plugin for gramps
 #
 # Copyright (C) 2001  Jesper Zedlitz
-# Copyright (C) 2004  Donald Allingham
+# Copyright (C) 2004-2006  Donald Allingham
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: CountAncestors.py,v 1.9.2.1 2005/09/27 04:54:17 rshura Exp $
+# $Id: CountAncestors.py 6842 2006-06-02 00:40:54Z rshura $
 
 "View/Number of ancestors"
 
@@ -30,7 +30,11 @@
 #------------------------------------------------------------------------
 import os
 from gettext import gettext as _
-from sets import Set
+from math import pow
+try:
+    set()
+except NameError:
+    from sets import Set as set
 
 #------------------------------------------------------------------------
 #
@@ -44,44 +48,59 @@ import gtk.glade
 # GRAMPS modules
 #
 #------------------------------------------------------------------------
-import Utils
-import Report
+from PluginUtils import register_report
+from ReportBase import CATEGORY_VIEW, MODE_GUI
+import ManagedWindow
 
 #------------------------------------------------------------------------
 #
 # 
 #
 #------------------------------------------------------------------------
-class CountAncestors:
+class CountAncestors(ManagedWindow.ManagedWindow):
     
-    def __init__(self,database,person):
-        
+    def __init__(self,dbstate,uistate,person):
+        self.title = _('Ancestors of "%s"') \
+                     % person.get_primary_name().get_name()
+
+        ManagedWindow.ManagedWindow.__init__(self,uistate,[],self.__class__)
+
+        database = dbstate.db        
         text = ""
         glade_file = "%s/summary.glade" % os.path.dirname(__file__)
         topDialog = gtk.glade.XML(glade_file,"summary","gramps")
         topDialog.signal_autoconnect({
-            "destroy_passed_object" : Utils.destroy_passed_object,
+            "destroy_passed_object" : self.close,
             })
-        thisgen = Set()
-        all = Set()
+        thisgen = set()
+        all = set()
         allgen = 0
+        total_theoretical = 0
         thisgen.add(person.get_handle())
         
-        title = _("Number of ancestors of \"%s\" by generation") % person.get_primary_name().get_name()
-        text += title + ':\n'
+
+        window = topDialog.get_widget("summary")
+        title = topDialog.get_widget("title")
+        self.set_window(window,title,self.title)
+
         thisgensize = 1
-        gen = 1
+        gen = 0
         while thisgensize > 0:
             thisgensize = 0
             if thisgen:
                 thisgensize = len( thisgen )
-                gen -= 1
+                gen += 1
+                theoretical = pow(2, ( gen - 1 ) )
+                total_theoretical += theoretical
+                percent = ( thisgensize / theoretical ) * 100
                 if thisgensize == 1 :
-                    text += _("Generation %d has 1 individual.\n") % (gen)
+                    text += _("Generation %d has 1 individual. (%3.2f%%)\n") \
+                            % (gen,percent)
                 else:
-                    text += _("Generation %d has %d individuals.\n") % (gen, thisgensize)
+                    text += _("Generation %d has %d individuals. (%3.2f%%)\n")\
+                            % (gen,thisgensize,percent)
             temp = thisgen
-            thisgen = Set()
+            thisgen = set()
             for person_handle in temp:
                 person = database.get_person_from_handle(person_handle)
                 family_handle = person.get_main_parents_family_handle()
@@ -96,28 +115,34 @@ class CountAncestors:
                         thisgen.add(mother_handle)
                         all.add(mother_handle)
             allgen += len(thisgen)
-	  
-        text += _("Total ancestors in generations %d to -1 is %d.\n") % (gen, allgen)
 
-        top = topDialog.get_widget("summary")
+        if( total_theoretical != 1 ):
+            percent = ( (allgen) / (total_theoretical-1) ) * 100
+        else:
+            percent = 0
+
+        text += _("Total ancestors in generations 2 to %d is %d. (%3.2f%%)\n")\
+                % (gen,allgen,percent)
+
         textwindow = topDialog.get_widget("textwindow")
-        topDialog.get_widget("title").set_text(title)
         textwindow.get_buffer().set_text(text)
-        top.show()
+        self.show()
+
+    def build_menu_names(self,obj):
+        return (self.title,None)
 
 #-------------------------------------------------------------------------
 #
 #
 #
 #-------------------------------------------------------------------------
-from PluginMgr import register_report
 register_report(
     name = 'count_ancestors',
-    category = Report.CATEGORY_VIEW,
+    category = CATEGORY_VIEW,
     report_class = CountAncestors,
     options_class = None,
-    modes = Report.MODE_GUI,
+    modes = MODE_GUI,
     translated_name = _("Number of ancestors"),
-    status = _("Beta"),
+    status = _("Stable"),
     description= _("Counts number of ancestors of selected person")
     )
