@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: EventCmp.py 7325 2006-09-24 04:37:59Z dallingham $
+# $Id: EventCmp.py 7951 2007-01-22 03:56:23Z dallingham $
 
 "Exploration/Compare individual events"
 
@@ -28,6 +28,7 @@
 #
 #------------------------------------------------------------------------
 import os
+import sys
 from gettext import gettext as _
 
 #------------------------------------------------------------------------
@@ -413,7 +414,8 @@ class DisplayChart(ManagedWindow.ManagedWindow):
 
         f.set_current_folder(os.getcwd())
         status = f.run()
-        name = unicode(f.get_filename())
+        name = unicode(f.get_filename(),
+                       sys.getfilesystemencoding())
         f.destroy()
 
         if status == gtk.RESPONSE_OK:
