@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: FindDupes.py 7102 2006-08-01 03:41:43Z dallingham $
+# $Id: FindDupes.py 7837 2006-12-22 19:24:46Z dallingham $
 
 "Database Processing/Merge people"
 
@@ -101,9 +101,6 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         self.update = callback
         self.use_soundex = 1
 
-        self.family_list = self.db.get_family_handles()[:]
-        self.person_list = self.db.get_person_handles(sort_handles=False)[:]
-
         base = os.path.dirname(__file__)
         self.glade_file = "%s/%s" % (base,"merge.glade")
         top = gtk.glade.XML(self.glade_file,"dialog","gramps")
@@ -187,6 +184,7 @@ class Merge(Tool.Tool,ManagedWindow.ManagedWindow):
         males = {}
         females = {}
 
+        self.person_list = self.db.get_person_handles(sort_handles=False)
         length = len(self.person_list)
 
         self.progress.set_pass(_('Pass 1: Building preliminary lists'),
