@@ -19,7 +19,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: GraphViz.py 7666 2006-11-22 04:43:33Z pez4brian $
+# $Id: GraphViz.py 7905 2007-01-14 22:09:14Z pez4brian $
 
 "Generate files/Relationship graph"
 
@@ -31,6 +31,7 @@
 import os
 from gettext import gettext as _
 from time import asctime
+import tempfile
 
 #------------------------------------------------------------------------
 #
@@ -125,14 +126,23 @@ class _options:
         ('',  "Descendants - Ancestors",    _("Descendants - Ancestors")),
     )
 
+gs_cmd = ""
+
 if os.sys.platform == "win32":
     _dot_found = Utils.search_for("dot.exe")
+    
+    if Utils.search_for("gswin32c.exe") == 1:
+        gs_cmd = "gswin32c.exe"
+    elif Utils.search_for("gswin32.exe") == 1:
+        gs_cmd = "gswin32.exe"
 else:
     _dot_found = Utils.search_for("dot")
+    
+    if Utils.search_for("gs") == 1:
+        gs_cmd = "gs"
 
-if Utils.search_for("epstopdf") == 1:
+if gs_cmd != "":
     _options.formats += (("pdf", "PDF", _("PDF"), "application/pdf"),)
-    _pdf_pipe = 'epstopdf -f -o=%s'
 
 #------------------------------------------------------------------------
 #
@@ -158,6 +168,7 @@ class GraphViz:
                      The option class carries its number, and the function
                      returning the list of filters.
         font       - Font to use.
+        fontsize   - Size of the font in points
         latin      - Set if text needs to be converted to latin-1
         arrow      - Arrow styles for heads and tails.
         showfamily - Whether to show family nodes.
@@ -220,6 +231,7 @@ class GraphViz:
         self.rankdir = options['rankdir']
         self.ratio = options['ratio']
         self.fontname = options['font']
+        self.fontsize = options['fontsize']
         self.colorize = options['color']
         if self.colorize == 'colored':
             self.colors = colored
@@ -295,7 +307,7 @@ size="%3.2f,%3.2f";
 rotate=%d;
 /* default node and edge attributes */
 nodesep=0.25;
-edge [syle=solid, arrowhead=%s arrowtail=%s];
+edge [style=solid, arrowhead=%s arrowtail=%s];
 """ %   (
         self.ratio,
         self.rankdir,
@@ -309,9 +321,10 @@ edge [syle=solid, arrowhead=%s arrowtail=%s];
         )
 
         if self.fontname:
-            font = 'fontname="%s"' % self.fontname
+            font = 'fontname="%s" ' % self.fontname
         else:
             font = ''
+        font += 'fontsize="%d"' % self.fontsize
         if self.colorize == 'filled':
             buffer += 'node [style=filled %s];\n' % font
         else:
@@ -578,6 +591,7 @@ class GraphVizOptions(ReportOptions):
         # Options specific for this report
         self.options_dict = {
             'font'       : "",
+            'fontsize'   : 14,
             'latin'      : 1,
             'arrow'      : 'd',
             'showfamily' : 1,
@@ -604,6 +618,8 @@ class GraphVizOptions(ReportOptions):
             'font'      : ("=str","Font to use in the report.",
                             [ "%s\t%s" % (item[0],item[1]) for item in _options.fonts ],
                             False),
+            'fontsize'  : ("=num","Font size (in points).",
+                            "Integer values"),
             'latin'     : ("=0/1","Needs to be set if font doesn't support unicode.",
                             ["Supports unicode","Supports only Latin1"],
                             True),
@@ -794,6 +810,14 @@ class GraphVizOptions(ReportOptions):
                                 "characters don't show, use FreeSans font. "
                                 "FreeSans is available from: "
                                 "http://www.nongnu.org/freefont/"))
+        
+        fontsize_adj = gtk.Adjustment(value=self.options_dict['fontsize'],
+                                      lower=8, upper=128, step_incr=1)
+        self.fontsize_sb = gtk.SpinButton(adjustment=fontsize_adj, digits=0)
+        dialog.add_frame_option(_("GraphViz Options"),
+                                _("Font size (in points)"),
+                                self.fontsize_sb,
+                                _("The font size, in points."))
 
         self.latin_cb = gtk.CheckButton(_("Output format/font requires text as latin-1"))
         self.latin_cb.set_active(self.options_dict['latin'])
@@ -914,6 +938,7 @@ class GraphVizOptions(ReportOptions):
         self.options_dict['pagesh'] = self.hpages_sb.get_value_as_int()
         self.options_dict['pagesv'] = self.vpages_sb.get_value_as_int()
         self.options_dict['showfamily'] = int(self.show_families_cb.get_active())
+        self.options_dict['fontsize'] = self.fontsize_sb.get_value_as_int()
         self.options_dict['incid'] = int(self.includeid_cb.get_active())
         self.options_dict['justyears'] = int(self.just_years_cb.get_active())
         self.options_dict['placecause'] = int(self.place_cause_cb.get_active())
@@ -1143,7 +1168,9 @@ class GraphVizGraphics(Report):
         self.doc = options_class.get_document()
 
         self.user_output = options_class.get_output()
-        self.junk_output = os.path.join(const.home_dir,"junk")
+        (handle,self.junk_output) = tempfile.mkstemp(".dot", "rel_graph" )
+        os.close( handle )
+
         self.the_format = self.options_class.handler.options_dict['gvof']
         self.the_font = self.options_class.handler.options_dict['font']
 
@@ -1155,26 +1182,31 @@ class GraphVizGraphics(Report):
 
     def end_report(self):
     	if self.the_format == "pdf":
-            command = ('dot -Tps %s | ' + _pdf_pipe + ' ; rm %s') % \
-                      (self.junk_output,self.user_output,self.junk_output)
-    	    os.system(command)
-    	else:
-    	    os.system('dot -T%s -o"%s" "%s"' %
-    	    	       (self.the_format,self.user_output,self.junk_output) )
-            os.remove(self.junk_output)
+            # Create a temporary Postscript file
+            (handle,tmp_ps) = tempfile.mkstemp(".ps", "rel_graph" )
+            os.close( handle )
 
-        if self.doc.print_req:
-            _apptype = None
-            for format in _options.formats:
-                if format[0] == self.the_format:
-                    _apptype = format[3]
-                    break
-            if _apptype:
-                try:
-                    app = Mime.get_application(_apptype)
-                    Utils.launch(app[0],self.user_output)
-                except:
-                    pass
+            # Generate Postscript using dot
+            command = 'dot -Tps -o"%s" "%s"' % ( tmp_ps, self.junk_output )
+            os.system(command)
+
+            paper = self.options_class.handler.get_paper()
+            # Add .5 to remove rounding errors.
+            width_pt = int( (paper.get_width_inches() * 72) + 0.5 )
+            height_pt = int( (paper.get_height_inches() * 72) + 0.5 )
+            
+            # Convert to PDF using ghostscript
+            command = '%s -q -sDEVICE=pdfwrite -dNOPAUSE -dDEVICEWIDTHPOINTS=%d -dDEVICEHEIGHTPOINTS=%d -sOutputFile="%s" "%s" -c quit' \
+                      % ( gs_cmd, width_pt, height_pt, self.user_output, tmp_ps )
+            os.system(command)
+
+            os.remove(tmp_ps)
+            
+        else:
+            os.system('dot -T%s -o"%s" "%s"' %
+                       (self.the_format,self.user_output,self.junk_output) )
+            
+        os.remove(self.junk_output)
 
         if self.doc.print_req:
             _apptype = None
