@@ -1,7 +1,7 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2000-2005  Martin Hawlisch, Donald N. Allingham
+# Copyright (C) 2000-2006  Martin Hawlisch, Donald N. Allingham
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: ImportvCard.py,v 1.2.2.2 2005/09/29 21:01:07 loshawlos Exp $
+# $Id: ImportvCard.py 6371 2006-04-19 22:59:33Z dallingham $
 
 "Import from vCard"
 
@@ -31,6 +31,14 @@ import re
 import time
 from gettext import gettext as _
 
+#------------------------------------------------------------------------
+#
+# Set up logging
+#
+#------------------------------------------------------------------------
+import logging
+log = logging.getLogger(".ImportVCard")
+
 #-------------------------------------------------------------------------
 #
 # GTK/GNOME Modules
@@ -48,6 +56,7 @@ import Errors
 import RelLib
 import const
 from QuestionDialog import ErrorDialog
+from PluginUtils import register_import
 
 #-------------------------------------------------------------------------
 #
@@ -61,11 +70,6 @@ def importData(database, filename, cb=None):
     except IOError,msg:
         ErrorDialog(_("%s could not be opened\n") % filename,str(msg))
         return
-    except:
-        ErrorDialog(_("%s could not be opened\n") % filename)
-        import DisplayTrace
-        DisplayTrace.DisplayTrace()
-        return
 
     try:
         status = g.parse_vCard_file()
@@ -73,11 +77,6 @@ def importData(database, filename, cb=None):
         errmsg = _("%s could not be opened\n") % filename
         ErrorDialog(errmsg,str(msg))
         return
-    except:
-        import DisplayTrace
-        DisplayTrace.DisplayTrace()
-        return
-    
 
 #-------------------------------------------------------------------------
 #
@@ -99,8 +98,7 @@ class VCardParser:
         return line
         
     def parse_vCard_file(self):
-        self.trans = self.db.transaction_begin()
-        self.trans.set_batch(True)
+        self.trans = self.db.transaction_begin("",batch=True)
         self.db.disable_signals()
         t = time.time()
         self.person = None
@@ -175,7 +173,7 @@ class VCardParser:
     def add_name(self, fields, data):
         data_fields = data.split(";")
         name = RelLib.Name()
-        name.set_type("Also Known As")
+        name.set_type(RelLib.NameType(RelLib.NameType.AKA))
         name.set_surname(data_fields[0])
         name.set_first_name(data_fields[1])
         if data_fields[2]:
@@ -187,7 +185,7 @@ class VCardParser:
 
     def add_title(self, fields, data):
         name = RelLib.Name()
-        name.set_type("Also Known As")
+        name.set_type(RelLib.NameType(RelLib.NameType.AKA))
         name.set_title(data)
         self.person.add_alternate_name(name)
 
@@ -208,7 +206,7 @@ class VCardParser:
 
     def add_birthday(self, fields, data):
         event = RelLib.Event()
-        event.set_name("Birth")
+        event.set_type(RelLib.EventType(RelLib.EventType.BIRTH))
         self.db.add_event(event,self.trans)
         self.person.set_birth_handle(event.get_handle())
 
@@ -222,7 +220,6 @@ class VCardParser:
 #
 #
 #-------------------------------------------------------------------------
-from PluginMgr import register_import
 _mime_type = const.app_vcard
 for mime in _mime_type:
     _filter = gtk.FileFilter()
