@@ -1,7 +1,7 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2003-2005  Donald N. Allingham
+# Copyright (C) 2003-2006  Donald N. Allingham
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 #
 # To see things still missing, search for "TODO"...
 #
-# $Id: StatisticsChart.py,v 1.23.2.11 2006/04/14 19:22:53 rshura Exp $
+# $Id: StatisticsChart.py 7296 2006-09-01 22:57:57Z rshura $
 
 """
 Statistics Chart report
@@ -34,7 +34,7 @@ Statistics Chart report
 #
 #------------------------------------------------------------------------
 import time
-from gettext import gettext as _
+from TransUtils import sgettext as _
 
 #------------------------------------------------------------------------
 #
@@ -50,16 +50,15 @@ import gtk
 #------------------------------------------------------------------------
 
 # Person and relation types
-from RelLib import Person, Family
+from RelLib import Person, FamilyRelType, EventType
 # gender and report type names
 import BaseDoc
-import Report
-import ReportUtils
-import ReportOptions
-import GenericFilter
-from DateHandler import displayer as _dd
+from PluginUtils import register_report
+from ReportBase import Report, ReportUtils, ReportOptions, \
+     CATEGORY_DRAW, MODE_GUI, MODE_BKI, MODE_CLI
+from Filters import GenericFilter, Rules
+import DateHandler
 from Utils import ProgressMeter
-from TransUtils import strip_context as __
 
 #------------------------------------------------------------------------
 #
@@ -94,7 +93,7 @@ class Extract:
         """Methods for extracting statistical data from the database"""
         # key, non-localized name, localized name, type method, data method
         self.extractors = {
-            'data_title':  ("Title", __("person|Title"),
+            'data_title':  ("Title", _("person|Title"),
                             self.get_person, self.get_title),
             'data_sname':  ("Surname", _("Surname"),
                             self.get_person, self.get_surname),
@@ -110,8 +109,6 @@ class Extract:
                             self.get_birth, self.get_month),
             'data_dmonth': ("Death month", _("Death month"),
                             self.get_death, self.get_month),
-            'data_dcause': ("Cause of death", _("Cause of death"),
-                             self.get_death, self.get_cause),
             'data_bplace': ("Birth place", _("Birth place"),
                             self.get_birth, self.get_place),
             'data_dplace': ("Death place", _("Death place"),
@@ -191,16 +188,9 @@ class Extract:
         if date:
             month = date.get_month()
             if month:
-                return [_dd._months[month]]
+                return [DateHandler.displayer._months[month]]
         return [_("Date(s) missing")]
 
-    def get_cause(self, event):
-        "return cause for given event"
-        cause = event.get_cause()
-        if cause:
-            return [cause]
-        return [_("Cause missing")]
-
     def get_place(self, event):
         "return place for given event"
         place_handle = event.get_place_handle()
@@ -227,16 +217,16 @@ class Extract:
     
     def get_person_age(self, person):
         "return age for given person, if alive"
-        death_handle = person.get_death_handle()
-        if not death_handle:
+        death_ref = person.get_death_ref()
+        if not death_ref:
             return [self.estimate_age(person)]
         return [_("Already dead")]
 
     def get_death_age(self, person):
         "return age at death for given person, if dead"
-        death_handle = person.get_death_handle()
-        if death_handle:
-            return [self.estimate_age(person, death_handle)]
+        death_ref = person.get_death_ref()
+        if death_ref:
+            return [self.estimate_age(person, death_ref.ref)]
         return [_("Still alive")]
 
     def get_event_ages(self, data):
@@ -255,7 +245,8 @@ class Extract:
         person, event_handles = data
         for event_handle in event_handles:
             event = self.db.get_event_from_handle(event_handle)
-            types.append(_(event.get_name()))
+            evtType = str(event.get_type())
+            types.append(evtType)
         if types:
             return types
         return [_("Events missing")]
@@ -289,9 +280,9 @@ class Extract:
         person, child_handles = data
         for child_handle in child_handles:
             child = self.db.get_person_from_handle(child_handle)
-            birth_handle = child.get_birth_handle()
-            if birth_handle:
-                ages.append(self.estimate_age(person, birth_handle))
+            birth_ref = child.get_birth_ref()
+            if birth_ref:
+                ages.append(self.estimate_age(person, birth_ref.ref))
             else:
                 errors.append(_("Birth missing"))
                 continue
@@ -321,16 +312,16 @@ class Extract:
 
     def get_birth(self, person):
         "return birth event for given person or None"
-        birth_handle = person.get_birth_handle()
-        if birth_handle:
-            return self.db.get_event_from_handle(birth_handle)
+        birth_ref = person.get_birth_ref()
+        if birth_ref:
+            return self.db.get_event_from_handle(birth_ref.ref)
         return None
     
     def get_death(self, person):
         "return death event for given person or None"
-        death_handle = person.get_death_handle()
-        if death_handle:
-            return self.db.get_event_from_handle(death_handle)
+        death_ref = person.get_death_ref()
+        if death_ref:
+            return self.db.get_event_from_handle(death_ref.ref)
         return None
     
     def get_child_handles(self, person):
@@ -338,8 +329,8 @@ class Extract:
         children = []
         for fam_handle in person.get_family_handle_list():
             fam = self.db.get_family_from_handle(fam_handle)
-            for child_handle in fam.get_child_handle_list():
-                children.append(child_handle)
+            for child_ref in fam.get_child_ref_list():
+                children.append(child_ref.ref)
         # TODO: it would be good to return only biological children,
         # but GRAMPS doesn't offer any efficient way to check that
         # (I don't want to check each children's parent family mother
@@ -353,12 +344,11 @@ class Extract:
         marriages = []
         for family_handle in person.get_family_handle_list():
             family = self.db.get_family_from_handle(family_handle)
-            if family.get_relationship() == Family.MARRIED:
-                for event_handle in family.get_event_list():
-                    if event_handle:
-                        event = self.db.get_event_from_handle(event_handle)
-                        if event.get_name() == "Marriage":
-                            marriages.append(event_handle)
+            if int(family.get_relationship()) == FamilyRelType.MARRIED:
+                for event_ref in family.get_event_ref_list():
+                    event = self.db.get_event_from_handle(event_ref.ref)
+                    if int(event.get_type()) == EventType.MARRIAGE:
+                        marriages.append(event_ref.ref)
         if marriages:
             return (person, marriages)
         return None
@@ -373,7 +363,9 @@ class Extract:
 
     def get_event_handles(self, person):
         "return list of event handles for given person or None"
-        events = person.get_event_list()
+        events = []
+        for event_ref in person.get_event_ref_list():
+            events.append(event_ref.ref)
 
         if events:
             return (person, events)
@@ -469,7 +461,7 @@ _Extract = Extract()
 # Statistics report
 #
 #------------------------------------------------------------------------
-class StatisticsChart(Report.Report):
+class StatisticsChart(Report):
 
     def __init__(self, database, person, options_class):
         """
@@ -484,11 +476,10 @@ class StatisticsChart(Report.Report):
 
         To see what the options are, check the options help in the options class.
         """
-        Report.Report.__init__(self,database,person,options_class)
+        Report.__init__(self,database,person,options_class)
     
         filter_num = options_class.get_filter_number()
         filters = options_class.get_report_filters(person)
-        filters.extend(GenericFilter.CustomFilters.get_filters())
         filterfun = filters[filter_num]
 
         options = options_class.handler.options_dict
@@ -757,12 +748,12 @@ class StatisticsChart(Report.Report):
 # Statistics report options
 #
 #------------------------------------------------------------------------
-class StatisticsChartOptions(ReportOptions.ReportOptions):
+class StatisticsChartOptions(ReportOptions):
     """
     Defines options and provides their handling interface.
     """
     def __init__(self,name, person_id=None):
-        ReportOptions.ReportOptions.__init__(self, name, person_id)
+        ReportOptions.__init__(self, name, person_id)
 
     def set_new_options(self):
     # Options specific for this report
@@ -842,24 +833,26 @@ class StatisticsChartOptions(ReportOptions.ReportOptions):
             name = 'PERSON'
             gramps_id = ''
     
-        all = GenericFilter.GenericFilter()
+        all = GenericFilter()
         all.set_name(_("Entire Database"))
-        all.add_rule(GenericFilter.Everyone([]))
+        all.add_rule(Rules.Person.Everyone([]))
 
-        des = GenericFilter.GenericFilter()
+        des = GenericFilter()
         des.set_name(_("Descendants of %s") % name)
-        des.add_rule(GenericFilter.IsDescendantOf([gramps_id, 1]))
+        des.add_rule(Rules.Person.IsDescendantOf([gramps_id, 1]))
 
-        ans = GenericFilter.GenericFilter()
+        ans = GenericFilter()
         ans.set_name(_("Ancestors of %s") % name)
-        ans.add_rule(GenericFilter.IsAncestorOf([gramps_id, 1]))
+        ans.add_rule(Rules.Person.IsAncestorOf([gramps_id, 1]))
 
-        com = GenericFilter.GenericFilter()
+        com = GenericFilter()
         com.set_name(_("People with common ancestor with %s") % name)
-        com.add_rule(GenericFilter.HasCommonAncestorWith([gramps_id]))
-
-        return [all, des, ans, com]
+        com.add_rule(Rules.Person.HasCommonAncestorWith([gramps_id]))
 
+        the_filters = [all, des, ans, com]
+        from Filters import CustomFilters
+        the_filters.extend(CustomFilters.get_filters('Person'))
+        return the_filters
     
     def add_user_options(self, dialog):
         """
@@ -970,17 +963,16 @@ class StatisticsChartOptions(ReportOptions.ReportOptions):
 # Register report/options
 #
 #------------------------------------------------------------------------
-from PluginMgr import register_report
-
 register_report(
     name = 'statistics_chart',
-    category = Report.CATEGORY_DRAW,
+    category = CATEGORY_DRAW,
     report_class = StatisticsChart,
     options_class = StatisticsChartOptions,
-    modes = Report.MODE_GUI | Report.MODE_BKI | Report.MODE_CLI,
+    modes = MODE_GUI | MODE_BKI | MODE_CLI,
     translated_name = _("Statistics Chart"),
     status = (_("Stable")),
     author_name="Eero Tamminen",
     author_email="",
-    description= _("Generates statistical bar and pie charts of the people in the database.")
+    description= _("Generates statistical bar and pie charts of the people in the database."),
+    require_active=False,
     )
