@@ -1,7 +1,7 @@
 #
 # Gramps - a GTK+/GNOME based genealogy program
 #
-# Copyright (C) 2003-2005  Donald N. Allingham
+# Copyright (C) 2003-2006  Donald N. Allingham
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: TimeLine.py,v 1.39.2.5 2005/11/06 04:59:23 dallingham Exp $
+# $Id: TimeLine.py 7231 2006-08-22 05:28:55Z rshura $
 
 """
 Timeline report
@@ -29,8 +29,7 @@ Timeline report
 # python modules
 #
 #------------------------------------------------------------------------
-from gettext import gettext as _
-from TransUtils import strip_context as __
+from TransUtils import sgettext as _
 
 #------------------------------------------------------------------------
 #
@@ -44,20 +43,22 @@ import gtk
 # GRAMPS modules
 #
 #------------------------------------------------------------------------
-from ReportUtils import pt2cm
-import Report
+from PluginUtils import register_report
+from ReportBase import Report, ReportUtils, ReportOptions, \
+     CATEGORY_DRAW, MODE_GUI, MODE_BKI, MODE_CLI
+pt2cm = ReportUtils.pt2cm
 import BaseDoc
-import GenericFilter
+from Filters import GenericFilter, Rules
 import Sort
-import ReportOptions
 from QuestionDialog import ErrorDialog
+import NameDisplay
 
 #------------------------------------------------------------------------
 #
 # TimeLine
 #
 #------------------------------------------------------------------------
-class TimeLine(Report.Report):
+class TimeLine(Report):
 
     def __init__(self,database,person,options_class):
         """
@@ -75,21 +76,20 @@ class TimeLine(Report.Report):
         filter    - Filter to be applied to the people of the database.
                     The option class carries its number, and the function
                     returning the list of filters.
-        title     - Title of the report displayed on top
         sort_func - function used to sort entries, that returns -1/0/1
                     when given two personal handles (like cmp).
                     The option class carries its number, and the function
                     returning the list of sort functions.
         """
 
-        Report.Report.__init__(self,database,person,options_class)
+        Report.__init__(self,database,person,options_class)
 
         filter_num = options_class.get_filter_number()
         filters = options_class.get_report_filters(person)
-        filters.extend(GenericFilter.CustomFilters.get_filters())
         self.filter = filters[filter_num]
 
-        self.title = options_class.handler.options_dict['title']
+        name = NameDisplay.displayer.display_formal(person)
+        self.title = _("Timeline Graph for %s") % name
 
         sort_func_num = options_class.handler.options_dict['sortby']
         sort_functions = options_class.get_sort_functions(Sort.Sort(database))
@@ -194,15 +194,17 @@ class TimeLine(Report.Report):
         
         for p_id in self.plist:
             p = self.database.get_person_from_handle(p_id)
-            b_id = p.get_birth_handle()
-            if b_id:
-                b = self.database.get_event_from_handle(b_id).get_date_object().get_year()
+            b_ref = p.get_birth_ref()
+            if b_ref:
+                birth = self.database.get_event_from_handle(b_ref.ref)
+                b = birth.get_date_object().get_year()
             else:
                 b = None
 
-            d_id = p.get_death_handle()
-            if d_id:
-                d = self.database.get_event_from_handle(d_id).get_date_object().get_year()
+            d_ref = p.get_death_ref()
+            if d_ref:
+                death = self.database.get_event_from_handle(d_ref.ref)
+                d = death.get_date_object().get_year()
             else:
                 d = None
 
@@ -291,15 +293,17 @@ class TimeLine(Report.Report):
 
         for p_id in self.plist:
             p = self.database.get_person_from_handle(p_id)
-            b_id = p.get_birth_handle()
-            if b_id:
-                b = self.database.get_event_from_handle(b_id).get_date_object().get_year()
+            b_ref = p.get_birth_ref()
+            if b_ref:
+                birth = self.database.get_event_from_handle(b_ref.ref)
+                b = birth.get_date_object().get_year()
             else:
                 b = None
 
-            d_id = p.get_death_handle()
-            if d_id:
-                d = self.database.get_event_from_handle(d_id).get_date_object().get_year()
+            d_ref = p.get_death_ref()
+            if d_ref:
+                death = self.database.get_event_from_handle(d_ref.ref)
+                d = death.get_date_object().get_year()
             else:
                 d = None
 
@@ -310,9 +314,14 @@ class TimeLine(Report.Report):
             if d:
                 low = min(low,d)
                 high = max(high,d)
-               
-        low = (low/10)*10
-        high = ((high+9)/10)*10
+
+        # round the dates to the nearest decade
+        low = int((low/10))*10
+        high = int(((high+9)/10))*10
+        
+        # Make sure the difference is a multiple of 50 so all year ranges land
+        # on a decade.
+        low -= 50 - ((high-low) % 50)
         
         if low == None:
             low = high
@@ -340,28 +349,25 @@ class TimeLine(Report.Report):
 # 
 #
 #------------------------------------------------------------------------
-class TimeLineOptions(ReportOptions.ReportOptions):
+class TimeLineOptions(ReportOptions):
 
     """
     Defines options and provides handling interface.
     """
 
     def __init__(self,name,person_id=None):
-        ReportOptions.ReportOptions.__init__(self,name,person_id)
+        ReportOptions.__init__(self,name,person_id)
 
     def set_new_options(self):
         # Options specific for this report
         self.options_dict = {
             'sortby'    : 0,
-            'title'     : '',
         }
         self.options_help = {
             'sortby'    : ("=num","Number of a sorting function",
                             [item[0] for item in 
                                     self.get_sort_functions(Sort.Sort(None))],
                             True),
-            'title'     : ("=str","Title string for the report",
-                            "Whatever String You Wish"),
         }
 
     def enable_options(self):
@@ -407,23 +413,26 @@ class TimeLineOptions(ReportOptions.ReportOptions):
             name = 'PERSON'
             gramps_id = ''
 
-        all = GenericFilter.GenericFilter()
+        all = GenericFilter()
         all.set_name(_("Entire Database"))
-        all.add_rule(GenericFilter.Everyone([]))
+        all.add_rule(Rules.Person.Everyone([]))
 
-        des = GenericFilter.GenericFilter()
+        des = GenericFilter()
         des.set_name(_("Descendants of %s") % name)
-        des.add_rule(GenericFilter.IsDescendantOf([gramps_id,1]))
+        des.add_rule(Rules.Person.IsDescendantOf([gramps_id,1]))
 
-        ans = GenericFilter.GenericFilter()
+        ans = GenericFilter()
         ans.set_name(_("Ancestors of %s") % name)
-        ans.add_rule(GenericFilter.IsAncestorOf([gramps_id,1]))
+        ans.add_rule(Rules.Person.IsAncestorOf([gramps_id,1]))
 
-        com = GenericFilter.GenericFilter()
+        com = GenericFilter()
         com.set_name(_("People with common ancestor with %s") % name)
-        com.add_rule(GenericFilter.HasCommonAncestorWith([gramps_id]))
+        com.add_rule(Rules.Person.HasCommonAncestorWith([gramps_id]))
 
-        return [all,des,ans,com]
+        the_filters = [all,des,ans,com]
+        from Filters import CustomFilters
+        the_filters.extend(CustomFilters.get_filters('Person'))
+        return the_filters
 
     def get_sort_functions(self,sort):
         return [
@@ -447,19 +456,10 @@ class TimeLineOptions(ReportOptions.ReportOptions):
 
         dialog.add_option(_('Sort by'),self.sort_menu)
 
-        self.title_box = gtk.Entry()
-        if self.options_dict['title']:
-            self.title_box.set_text(self.options_dict['title'])
-        else:
-            self.title_box.set_text(dialog.get_header(dialog.person.get_primary_name().get_name()))
-        self.title_box.show()
-        dialog.add_option(__('report|Title'),self.title_box)
-
     def parse_user_options(self,dialog):
         """
         Parses the custom options that we have added.
         """
-        self.options_dict['title'] = unicode(self.title_box.get_text())
         self.options_dict['sortby'] = self.sort_menu.get_active()
 
 #------------------------------------------------------------------------
@@ -467,14 +467,12 @@ class TimeLineOptions(ReportOptions.ReportOptions):
 #
 #
 #------------------------------------------------------------------------
-from PluginMgr import register_report
-
 register_report(
     name = 'timeline',
-    category = Report.CATEGORY_DRAW,
+    category = CATEGORY_DRAW,
     report_class = TimeLine,
     options_class = TimeLineOptions,
-    modes = Report.MODE_GUI | Report.MODE_BKI | Report.MODE_CLI,
+    modes = MODE_GUI | MODE_BKI | MODE_CLI,
     translated_name = _("Timeline Graph"),
     status = _("Stable"),
     author_name = "Donald N. Allingham",
