@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: WriteCD.py 6860 2006-06-04 23:23:10Z pez4brian $
+# $Id: WriteCD.py 7951 2007-01-22 03:56:23Z dallingham $
 
 "Export to CD (nautilus)"
 
@@ -28,6 +28,7 @@
 #
 #-------------------------------------------------------------------------
 import os
+import sys
 from cStringIO import StringIO
 from gettext import gettext as _
 
@@ -234,7 +235,8 @@ class PackageWriter:
                 pass
 
             def fs_ok_clicked(obj):
-                newfile = fs_top.get_filename()
+                newfile = unicode(fs_top.get_filename(),
+                                  sys.getfilesystemencoding())
                 if os.path.isfile(newfile):
                     self.copy_file(newfile,'burn:///%s/%s' % (base,obase))
                     ntype = Mime.get_type(newfile)
