@@ -18,7 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
-# $Id: WritePkg.py 7299 2006-09-02 01:47:08Z pez4brian $
+# $Id: WritePkg.py 7993 2007-01-27 05:30:15Z dallingham $
 
 "Export to GRAMPS package"
 
@@ -29,6 +29,7 @@
 #-------------------------------------------------------------------------
 import time
 import os
+import sys
 import tarfile
 from cStringIO import StringIO
 from gettext import gettext as _
@@ -139,7 +140,8 @@ class PackageWriter:
                 pass
 
             def fs_ok_clicked(obj):
-                name = fs_top.get_filename()
+                name = unicode(fs_top.get_filename(),
+                               sys.getfilesystemencoding())
                 if os.path.isfile(name):
                     archive.add(name)
 
@@ -160,26 +162,26 @@ class PackageWriter:
             filename = mobject.get_path()
             if os.path.isfile(filename):
                 archive.add(filename)
-            else:
-                # File is lost => ask what to do
-                if missmedia_action == 0:
-                    mmd = MissingMediaDialog(
-                        _("Media object could not be found"),
-                        _("%(file_name)s is referenced in the database, "
-                          "but no longer exists. The file may have been "
-                          "deleted or moved to a different location. " 
-                          "You may choose to either remove the reference "
-                          "from the database, keep the reference to the "
-                          "missing file, or select a new file."
-                          ) % { 'file_name' : filename },
-                        remove_clicked, leave_clicked, select_clicked)
-                    missmedia_action = mmd.default_action
-                elif missmedia_action == 1:
-                    remove_clicked()
-                elif missmedia_action == 2:
-                    leave_clicked()
-                elif missmedia_action == 3:
-                    select_clicked()
+#             else:
+#                 # File is lost => ask what to do
+#                 if missmedia_action == 0:
+#                     mmd = MissingMediaDialog(
+#                         _("Media object could not be found"),
+#                         _("%(file_name)s is referenced in the database, "
+#                           "but no longer exists. The file may have been "
+#                           "deleted or moved to a different location. " 
+#                           "You may choose to either remove the reference "
+#                           "from the database, keep the reference to the "
+#                           "missing file, or select a new file."
+#                           ) % { 'file_name' : filename },
+#                         remove_clicked, leave_clicked, select_clicked)
+#                     missmedia_action = mmd.default_action
+#                 elif missmedia_action == 1:
+#                     remove_clicked()
+#                 elif missmedia_action == 2:
+#                     leave_clicked()
+#                 elif missmedia_action == 3:
+#                     select_clicked()
         
         # Write XML now
         g = StringIO()
